Linux
-----

#### Prerequisites

Under Debian and its derivatives, install required libraries by typing in terminal:
```bash
apt-get install cmake g++
apt-get install libopencv-dev
apt-get install libboost-all-dev
apt-get install qtbase5-dev qtbase5-dev-tools qttools5-dev qtscript5-dev
apt-get install xvfb mesa-utils libgl1-mesa-glx libgl1-mesa-dri fonts-cantarell
apt-get install python3-numpy python3-pandas python3-matplotlib python3-pil
```

#### Compilation

Download the latest source code and then in terminal type:
```bash
tar xfz shuttletracker-VERSION-source.tar.gz
mkdir shuttletracker-VERSION-build
cmake -S shuttletracker-VERSION-source \
      -B shuttletracker-VERSION-build  \
      -DCMAKE_BUILD_TYPE=Release       \
      -DCMAKE_INSTALL_PREFIX=/opt/local
cmake  --build shuttletracer-VERSION-build  --config Release  --parallel 4
(cd shuttletracer-VERSION-build  &&  make install)
chmod 666 /opt/local/share/shuttletracker/config/settings.ini
```

If OpenCV resides in a non-standrd location, add this argument to first cmake
invocation: `-DOpenCV_DIR=PATH_TO_YOUR_OPENCV/lib/cmake/opencv4`. If Boost resides
in a non-standard location, add `-DBOOST_ROOT=PATH_TO_YOUR_BOOST/boost/include`


#### Building a container

A "recipe" file to build an Ubuntu-based singularity container has been
provided in source code subdirectory distrib/linux. To build a container
using the recipe file, type:
````bash
  sudo singularity build --sandbox st SINGULARITY_RECIPE_FILE
````
ShuttleTracker can be launched within the container with:
````bash
  singularity exec --writable st /opt/local/bin/ShuttleTracker
````
or simply with:
````bash
  singularity run --app shuttletracker --writable st
````


Windows
-------

#### Prerequisites for building with Microsoft Visual C++ Compiler

* Download and install GPL-licensed Qt libraries and optionally QtCreator
  (<https://www.qt.io/download>).
* Download and extract precompiled OpenCV libraries (version **4**.x.x)
  (<https://opencv.org/releases>).
* Download and extract precompiled Boost C++ libraries (any recent version)
  (<https://sourceforge.net/projects/boost/files/boost-binaries>).

Remember to download OpenCV and Boost libraries with their "msvc" version 
that is appropriate for your Visual Studio version (e.g., VS2015: msvc-14.0, VS2017: msvc-14.1)

#### Prerequisites for building with GNU g++ within MinGW-w64

* Download and install MinGW-w64 (<http://mingw-w64.org>).
* Using MSYS2 terminal, update package info:
````bash
  pacman -Syyu
````
and install required libraries:
````bash
  pacman -Syu mingw-w64-x86_64-cmake
  pacman -Syu mingw-w64-x86_64-gcc
  pacman -Syu mingw-w64-x86_64-qt5
  pacman -Syu mingw-w64-x86_64-boost
  pacman -Syu mingw-w64-x86_64-opencv
````

#### Compiling with Microsoft Visual C++ Compiler using QMake and QtCreator

Open ShuttleTracker.pro file, make sure that paths to OpenCV and Boost libraries
are correct in the win32-msvc variant, and build your project in the Release mode.


#### Compiling with Microsoft Visual C++ Compiler using MinGW's CMake

In MinGW-w64 64-bit terminal:

````bash
  cmake  -B build64_vc141  -S shuttletracker-VERSION-source         \
    -G "Visual Studio 15 2017"  -A x64  -DCMAKE_BUILD_TYPE=Release  \
    -DMSVC_VERSION_DIR="C:/Program\ Files\ (x86)/Microsoft\ Visual\ Studio/2017/Community/VC/Tools/MSVC/14.16.27023" \
    -DCMAKE_PREFIX_PATH="/c/Qt/5.12.6/msvc2017_64/lib/cmake"                                                         \
    -DCMAKE_MODULE_PATH="/c/Boost/1.72.0/lib64-msvc-14.1/cmake/Boost-1.72.0;/c/OpenCV/4.2.0/build/x64/vc14/lib"      \
    -DBOOST_ROOT=/c/Boost/1.72.0 -DBOOST_LIBRARYDIR=/c/Boost/1.72.0/lib64-msvc-14.1 -DBoost_ARCHITECTURE="-x64"      \
    -DOpenCV_DIR=/c/OpenCV/4.2.0/build
  cmake  --build build64_vc141  --config Release
````
(Make sure that paths given in the command above are correct.)


#### Compiling with GNU g++ under MinGW-w64 using MinGW's CMake

In MinGW-w64 64-bit terminal:

````bash
  cmake -B build64_mingw  -S shuttletracker-VERSION-source  -G "MSYS Makefiles"  -DCMAKE_BUILD_TYPE=Release
  cmake  --build build64_mingw  --config Release  --parallel 4
````

#### Installer

To be able to create an installer, download and install NSIS
(Nullsoft Scriptable Install System, <https://sourceforge.net/projects/nsis>).
Check whether the file of your choice from `distrib/windows/make_installer_*.nsi`
has correct links to your just compiled executable and the C++ redistributable
toolkit (`vc_redist.*.exe`), and generate an installer in NSIS.


macOS
-----

Building a Mac drag-and-drop "app" does _not_ require XCode
(but nevertheless requires a working C++ compiler).
It's a multi-step process that is automated by the script
`distrib/macos/build_macos_release.sh`.
To use it, you should have MacPorts (<https://www.macports.org>)
installed and configured.
