# ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
#
# Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU GPL version 3. See LICENSE.md for details.


TARGET = ShuttleTracker
TEMPLATE = app

VERSION = 1.5.3
DEFINES += APP_NAME=\\\"$$TARGET\\\"
DEFINES += APP_VERSION=\\\"$$VERSION\\\"
win32 {
QMAKE_TARGET_PRODUCT     = "ShuttleTracker"
QMAKE_TARGET_DESCRIPTION = "ShuttleTracker, a semi-automated cell tracker"
QMAKE_TARGET_COMPANY     = "IPPT PAN, Warsaw"
QMAKE_TARGET_COPYRIGHT   = "Marek Kochanczyk <mkochan:at:ippt.pan.pl>"
}

lessThan(QT_MAJOR_VERSION, 5) { error(This software requires Qt 5.) }
equals(QT_MAJOR_VERSION, 5) {
  lessThan(QT_MINOR_VERSION, 15) {
      error(This software requires Qt version 5.15 or later.)
} }

QT += core gui widgets concurrent network qml
win32 { QT += winextras }

DEFINES += USE_QT

!win32 {
DEFINES += SHUTTLETRACKER_SHARED_DIR=\\\"/tmp/local/shuttletracker/share\\\"
}

INCLUDEPATH += source/
INCLUDEPATH += source/GUI/Support  # generated ui_AnyWindow.h requires some HPPs
INCLUDEPATH += contrib/

SOURCES += \
    source/GUI/main.cpp                                            \
    source/GUI/Windows/MainWindow.cpp                              \
    source/GUI/Windows/MainWindow_menuFile.cpp                     \
    source/GUI/Windows/MainWindow_menuView.cpp                     \
    source/GUI/Windows/MainWindow_menuScript.cpp                   \
    source/GUI/Windows/MainWindow_menuTools.cpp                    \
    source/GUI/Windows/MainWindow_menuPreferences.cpp              \
    source/GUI/Windows/MainWindow_menuWindow.cpp                   \
    source/GUI/Windows/MainWindow_menuHelp.cpp                     \
    source/GUI/Windows/MainWindow_toolboxImageMasking.cpp          \
    source/GUI/Windows/MainWindow_toolboxNucleiDetection.cpp       \
    source/GUI/Windows/MainWindow_toolboxNucleiDetection_input.cpp \
    source/GUI/Windows/MainWindow_toolboxNucleiEditing.cpp         \
    source/GUI/Windows/MainWindow_toolboxPerinuclei.cpp            \
    source/GUI/Windows/MainWindow_toolboxRegionsEditing.cpp        \
    source/GUI/Windows/MainWindow_toolboxQuantification.cpp        \
    source/GUI/Windows/MainWindow_toolboxTracking.cpp              \
    source/GUI/Windows/MainWindow_toolboxTracksEditing.cpp         \
    source/GUI/Windows/MainWindow_setup.cpp                        \
    source/GUI/Windows/MainWindow_image_loading.cpp                \
    source/GUI/Windows/MainWindow_drawing.cpp                      \
    source/GUI/Windows/MainWindow_editing.cpp                      \
    source/GUI/Windows/MainWindow_logging.cpp                      \
    source/GUI/Windows/MainWindow_navigation.cpp                   \
    source/GUI/Windows/MainWindow_persistence.cpp                  \
    source/GUI/Windows/MainWindow_scripting.cpp                    \
    source/GUI/Windows/MainWindow_unixSignals.cpp                  \
    source/GUI/Windows/MainWindow_view.cpp                         \
    source/GUI/Windows/MainWindow_aux.cpp                          \
    source/GUI/Windows/HelpWindow.cpp                              \
    source/GUI/Windows/LogWindow.cpp                               \
    source/GUI/Windows/ScriptWindow.cpp                            \
    source/GUI/Windows/AboutDialog.cpp                             \
    source/GUI/Support/PixmapCache.cpp                             \
    source/GUI/Support/ZoomableGraphicsView.cpp                    \
    source/GUI/Support/Outline.cpp                                 \
    source/GUI/Support/RichTextDialog.cpp                          \
    source/GUI/Support/Track.cpp                                   \
    source/GUI/Support/TracksModel.cpp                             \
    source/GUI/Support/ScriptEditor.cpp                            \
    source/GUI/Support/ScriptHighlighter.cpp                       \
    source/GUI/Support/LineNumberArea.cpp                          \
    source/Image/ErrorHandling.cpp                                 \
    source/Image/ImageChannel.cpp                                  \
    source/Image/ImageSequenceChannelInfo.cpp                      \
    source/Image/ImageSequenceFilenamePattern.cpp                  \
    source/Image/ProgressReporter.cpp                              \
    source/Image/SharedUtilities.cpp                               \
    source/Image/ImageSequence.cpp                                 \
    source/Image/ImageOperations.cpp                               \
    source/Image/ImageDecomposition.cpp                            \
    source/Image/NucleiDetection.cpp                               \
    source/Image/NucleiDetectionStained.cpp                        \
    source/Image/NucleiDetectionNonstained.cpp                     \
    source/Image/Perinuclei.cpp                                    \
    source/Image/Quantification.cpp                                \
    contrib/tinyxml2/tinyxml2.cpp

HEADERS += \
    source/common.hpp                             \
    source/GUI/GUICommon.hpp                      \
    source/GUI/Windows/MainWindow.hpp             \
    source/GUI/Windows/MainWindow_aux.hpp         \
    source/GUI/Windows/AboutDialog.hpp            \
    source/GUI/Windows/HelpWindow.hpp             \
    source/GUI/Windows/LogWindow.hpp              \
    source/GUI/Windows/ScriptWindow.hpp           \
    source/GUI/Support/PixmapCache.hpp            \
    source/GUI/Support/ZoomableGraphicsView.hpp   \
    source/GUI/Support/Outline.hpp                \
    source/GUI/Support/RichTextDialog.hpp         \
    source/GUI/Support/Track.hpp                  \
    source/GUI/Support/TracksModel.hpp            \
    source/GUI/Support/ScriptEditor.hpp           \
    source/GUI/Support/ScriptHighlighter.hpp      \
    source/GUI/Support/LineNumberArea.hpp         \
    source/Image/ImageCommon.hpp                  \
    source/Image/ErrorHandling.hpp                \
    source/Image/ImageChannel.hpp                 \
    source/Image/ImageSequenceChannelInfo.hpp     \
    source/Image/ImageSequenceFilenamePattern.hpp \
    source/Image/ProgressReporter.hpp             \
    source/Image/Types.hpp                        \
    source/Image/ImageSequence.hpp                \
    source/Image/ImageOperations.hpp              \
    source/Image/ImageDecomposition.hpp           \
    source/Image/NucleiDetection.hpp              \
    source/Image/NucleiDetectionStained.hpp       \
    source/Image/NucleiDetectionNonstained.hpp    \
    source/Image/Perinuclei.hpp                   \
    source/Image/Quantification.hpp               \
    source/Image/SharedUtilities.hpp              \
    contrib/gtest/FloatingPoint.hpp               \
    contrib/tinyxml2/tinyxml2.h

FORMS += \
    source/GUI/Windows/MainWindow.ui   \
    source/GUI/Windows/ScriptWindow.ui \
    source/GUI/Windows/HelpWindow.ui   \
    source/GUI/Windows/AboutDialog.ui

OTHER_FILES += \
    resources/darkorange.css     \
    resources/fixfusion.css      \
    resources/img/checkbox.png   \
    resources/img/handle.png     \
    resources/img/downarrow.png  \
    resources/icons/shuttletracker.ico \
    doc/Manual.html

RESOURCES += \
    resources/resources.qrc

CONFIG += c++17

QMAKE_CXXFLAGS_RELEASE += -DNDEBUG

linux {
#  QMAKE_CXX = g++-12
# QMAKE_LINK = g++-12
#
# QMAKE_CXXFLAGS += -fsanitize=address
# QMAKE_LFLAGS   += -fsanitize=address
# QMAKE_CXXFLAGS += -fsanitize=memory -fsanitize-memory-track-origins
# QMAKE_LFLAGS   += -fsanitize=memoy  -fsanitize-memory-track-origins

  QMAKE_CXXFLAGS += -isystem $$(HOME)/locals/boost/include
  QMAKE_CXXFLAGS += -isystem $$(HOME)/locals/opencv/include/opencv4
  QMAKE_LFLAGS   += -L$$(HOME)/locals/opencv/lib
  QMAKE_LFLAGS   += -L$$(HOME)/locals/boost/lib

  QMAKE_CXXFLAGS -= -std=c++0x
  QMAKE_CXXFLAGS += -std=c++17 -pipe
  QMAKE_CXXFLAGS += -Wall -Wextra -Wpedantic -Wfloat-equal -Wwrite-strings \
                    -Wunreachable-code -Wshadow -Wswitch-enum -Wno-inline  \
                    -Wnon-virtual-dtor

  QMAKE_CXXFLAGS_DEBUG   += -ggdb -O0
  QMAKE_CXXFLAGS_RELEASE -= -O2
  QMAKE_CXXFLAGS_RELEASE += -O3 -Wno-inline -flto -march=native
  QMAKE_LFLAGS_RELEASE   -= -O1 -Wl,-O1
  QMAKE_LFLAGS_RELEASE   += -O3             -flto -march=native
  LIBS += -lboost_filesystem -lboost_system -lboost_regex   \
          -lopencv_highgui -lopencv_imgcodecs -lopencv_core \
          -lopencv_imgproc -lopencv_photo -lopencv_ml \
          -ltbb
}

win32-msvc {
  QMAKE_CXXFLAGS += -FI '"C:\Program Files (x86)\Microsoft Visual Studio\2019\Community\VC\Tools\MSVC\14.29.30133\include\ciso646"'
  QMAKE_CXXFLAGS_DEBUG   += -Zi   -DEBUG
  QMAKE_CXXFLAGS_RELEASE += -O2 -DNDEBUG
  INCLUDEPATH += "C:\Boost\1.81.0" \
                 "C:\OpenCV\4.7.0\build\include"
  LIBPATH += "C:\Boost\1.81.0\lib64-msvc-14.2"    \
             "C:\OpenCV\4.7.0\build\x64\vc16\bin" \
             "C:\OpenCV\4.7.0\build\x64\vc16\lib"
  LIBS += -lboost_filesystem-vc142-mt-x64-1_81 \
          -lboost_system-vc142-mt-x64-1_81     \
          -lboost_regex-vc142-mt-x64-1_81      \
          -lopencv_world470
  DISTFILES += resources/win_resources.rc
  RC_ICONS  =  resources/icons/shuttletracker.ico
}

## -- OUTDATED
#
#win32-mingw {
#  QMAKE_CXXFLAGS += -std=c++11 -pipe -Wall -Wextra -Wpedantic
#  QMAKE_CXXFLAGS += -isystem "C:\Boost\1.57.0\include" \
#                    -isystem "C:\OpenCV\2.4.8\include"
#  QMAKE_CXXFLAGS_RELEASE += -O3 -Wno-inline
#  QMAKE_CXXFLAGS_DEBUG += -ggdb -O1
#  LIBPATH += "C:\Boost\1.57.0\lib-32bit" \
#             "C:\OpenCV\2.4.8\bin"       \
#             "C:\OpenCV\2.4.8\lib"
#  LIBS += -lboost_filesystem-mgw49-mt-1_57 \
#          -lboost_system-mgw49-mt-1_57     \
#          -lboost_regex-mgw49-mt-1_57      \
#          -lopencv_core248                 \
#          -lopencv_imgproc248              \
#          -lopencv_highgui248              \
#          -lopencv_photo248                \
#          -lopencv_ml248
#  DISTFILES += resources/win_resources.rc
#  RC_ICONS  =  resources/icons/shuttletracker.ico
#}
