#!/usr/bin/env python2.7

from __future__ import print_function
import sys
import os.path
import subprocess
import shutil
import glob

mac_otool_path = '/usr/bin/otool'
mac_install_name_tool_path = '/usr/bin/install_name_tool'
macports_sysroot_libs_prefix = '/opt/local/lib'  # works as lib* to account for both lib and libexec

binaries_to_resolve = set(glob.glob('*'))  #set(sys.argv[1:])
binaries_resolved   = set()

while len(binaries_to_resolve) > 0:

  b = list(binaries_to_resolve)[0]
  print(',-----',b,'-----')

  if os.path.islink(b):
    print("WHOOPS, that's LINK:", b)
    sys.exit(1)

  deps = [ l.split()[0] for l in subprocess.check_output([mac_otool_path, '-L', b]).split('\n\t')[1:] \
           if macports_sysroot_libs_prefix in l or not os.sep in l ]
  for d_path in deps:

    d_name = d_path.split(os.path.sep)[-1] if os.path.sep in d_path else d_path
    if d_path == d_name and d_name in binaries_resolved:
      print('| Skipping already resolved:',d_name)
      continue

    if d_path == d_name:
      print('| Already here',d_path)
    else:
      shutil.copyfile(d_path, d_name)
      print('| Copied',d_path)
    # SPECIAL FIX for an exceptionally misleading self-reference failure in Qwt
    if d_name != 'ShuttleTracker':
      binaries_to_resolve.add(d_name)

    # SPECIAL FIX for an exceptionally misleading self-reference failure in Qwt
    using_extra_fix = False
    if d_name in b:
      subprocess.check_output([mac_install_name_tool_path,'-id',         '@rpath/Resources/'+d_name,b])
    subprocess.check_output([mac_install_name_tool_path,'-change',d_path,'@rpath/Resources/'+d_name,b])
    print('| Resolved',b,'~~=>',d_name, '[extra fix]' if using_extra_fix else '')

  binaries_resolved.add(b)
  binaries_to_resolve.remove(b)


print('Binaries NOT resolved:')
if len(binaries_to_resolve) == 0:
  print('  (none)')
else:
  for b in binaries_to_resolve: 
    print(' ',b)

print('Binaries resolved:')
for b in binaries_resolved:
  print(' ',b)
