#include "ImageSequenceReaderThread.hpp"

#include <iostream>
#include <QDebug>//tmp
#include <QMutexLocker>
#include <ImageSequence.hpp>

//QMutex ImageSequenceReaderThread::mutex_;


ImageSequenceReaderThread::ImageSequenceReaderThread(
    QObject* parent,
    ImageSequenceType* is,
    QDir const& dir)
: QThread{parent},
  image_sequence_{is},
  dir_{dir},
  timer_{this},
  stop_requested_{false}
{
  connect(&timer_, SIGNAL(timeout()), this, SLOT(periodically()));
}


void ImageSequenceReaderThread::run()
{
  emit startProgress();
  //QMutexLocker lock(&mutex_);

  timer_.start(200); // ms

  try {
    image_sequence_->read_directory(dir_.path().toStdString());
  } catch (std::runtime_error &e) {
    timer_.stop();
    emit stopProgress();
    emit readingError(e.what());
    QThread::exit(1);
    return;
  } catch (...) {
    //qDebug() << image_sequence_;
    timer_.stop();
    emit stopProgress();
    emit readingError("Unknown problem.");
    QThread::exit(1);
    return;
  }

  emit stopProgress();
}


void ImageSequenceReaderThread::exit(int retcode)
{
  stop_requested_ = true;
  std::cout << "Done via exit." << std::endl;
  timer_.stop();
  emit stopProgress();
  QThread::exit(retcode);
}


void ImageSequenceReaderThread::periodically()
{
  sayProgress();
}


void ImageSequenceReaderThread::sayProgress()
{
  std::cout << "Alive " << this << std::endl;
  if (image_sequence_->busy()) {
    emit progressMaxValue(image_sequence_->progress_target());
    emit progressCurValue(image_sequence_->progress());
  }
}
