// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#include "RichTextDialog.hpp"
#include <QTextEdit>
#include <QVBoxLayout>
#include <QFont>
#include <QString>

RichTextDialog::RichTextDialog(int width, int height, QWidget* parent)
: QDialog{parent},
  layout_{new QVBoxLayout(this)},
  text_{new QTextEdit(this)},
  text_end_anchor_{"__LOG_END__"}
{
  QFont font{"Monospace"};
  font.setStyleHint(QFont::TypeWriter);
  text_->setFont(font);
  text_->setReadOnly(true);
  text_->setMinimumWidth(width);
  text_->setMinimumHeight(height);
  text_->setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOn);
  text_->setFrameStyle(0);

  layout()->addWidget(text_);
  layout()->setMargin(5);  // absent in Qt 6
  setLayout(layout_);
}


void RichTextDialog::setText(QString const& text)
{
  QString text_hl = text;
  text_hl.replace("\n","<br>")
         .replace("Info:",   "<font color=\"#aaddaa\">Info:</font>")
         .replace("Warning:","<font color=\"#cfcfaa\">Warning:</font>")
         .replace("Error:",  "<font color=\"#eeaaaa\">Error:</font>");
  text_hl +=   QString("<br/>")
             + "<a name=\"" + text_end_anchor_ + "\""
             + " href=\"#" + text_end_anchor_ + "\""
             + " style=\"text-decoration: none\">"
             + "&nbsp;</a>";
  text_->setHtml(text_hl);
  text_->scrollToAnchor(text_end_anchor_);
}
