// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#ifndef SHUTTLETRACKER_GUI_SUPPORT_SCRIPTEDITOR_HPP
#define SHUTTLETRACKER_GUI_SUPPORT_SCRIPTEDITOR_HPP

#include "GUI/GUICommon.hpp"
#include "GUI/Support/ScriptHighlighter.hpp"
#include <QPlainTextEdit>
#include <QObject>

QT_BEGIN_NAMESPACE
class QProgressBar;
class QPaintEvent;
class QResizeEvent;
class QSize;
class QWidget;
class QAbstractItemModel;
class QCompleter;
QT_END_NAMESPACE
class LineNumberArea;

class ScriptEditor : public QPlainTextEdit {

  Q_OBJECT

public:
  ScriptEditor(QWidget* parent=nullptr, bool show_line_numbers=true);
  ~ScriptEditor() OVERRIDE;

  void setCurrentLineHighlighting(bool);
  void lineNumberAreaPaintEvent(QPaintEvent* event);
  int lineNumberAreaWidth();
  void hideLineNumbersArea() { showing_line_numbers_ = false; }
  void setCompleter(QCompleter* c);
  QCompleter* getCompleter() const { return completer_; }
  void setCompleterWordlist(QStringList const& words);
  QAbstractItemModel* modelFromFile(QString const& fileName);
  void goToLine(int);

protected:
  void resizeEvent(QResizeEvent* event) OVERRIDE;
  void keyPressEvent(QKeyEvent* e) OVERRIDE;  // compl
  void focusInEvent(QFocusEvent* e) OVERRIDE;  // compl

private slots:
  void updateLineNumberAreaWidth(int newBlockCount);
  void highlightCurrentLine();
  void updateLineNumberArea(QRect const&, int);
  void insertCompletion(QString const& completion);

private:
  QString textUnderCursor() const; // compl

private:
  QWidget* line_number_area_;
  QFont line_number_area_font_;
  ScriptHighlighter* highlighter_;
  QCompleter* completer_;
  bool current_line_highlighting_enabled_;
  bool showing_line_numbers_ = true;
}; // class ScriptEditor


#endif // !define(SHUTTLETRACKER_GUI_SUPPORT_SCRIPTEDITOR_HPP)
