// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#ifndef SHUTTLETRACKER_GUI_SUPPORT_SCRIPTHIGHLIGHTER_HPP
#define SHUTTLETRACKER_GUI_SUPPORT_SCRIPTHIGHLIGHTER_HPP

#include "GUI/GUICommon.hpp"
#include <QRegularExpression>
#include <QHash>
#include <QSyntaxHighlighter>
#include <QTextCharFormat>

QT_BEGIN_NAMESPACE
class QTextDocument;
QT_END_NAMESPACE


class ScriptHighlighter : public QSyntaxHighlighter {

  Q_OBJECT

public:
  ScriptHighlighter(QTextDocument* parent = nullptr);

protected:
  void highlightBlock(const QString &text) OVERRIDE;

private:
  struct HighlightingRule
  {
    QRegularExpression pattern;
    QTextCharFormat format;
  };

  QVector<HighlightingRule> highlightingRules;

  QRegularExpression commentStartExpression;
  QRegularExpression commentEndExpression;

  QTextCharFormat primaryKeywordFormat;   // JS keywords
  QTextCharFormat secondaryKeywordFormat; // API functions
  QTextCharFormat tertiaryKeywordFormat;  // API parameters
  QTextCharFormat quaternaryKeywordFormat;// API special
  QTextCharFormat quotationFormat;
  QTextCharFormat singleLineCommentFormat;
  QTextCharFormat multiLineCommentFormat;
}; // class ScriptHighlighter

#endif // !defined(SHUTTLETRACKER_GUI_SUPPORT_SCRIPTHIGHLIGHTER_HPP)
