// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#include "TracksModel.hpp"


int TracksModel::rowCount(QModelIndex const& /*parent*/) const
{  return tracks_->size();  }


int TracksModel::columnCount(QModelIndex const& /*parent*/) const
{  return 3;  }  // 1: number of segments,
                 // 2: segments range
                 // 3: revision status


QVariant TracksModel::data(QModelIndex const& index, int role) const
{
  if (not index.isValid()) { return QVariant(); }
  if (index.row() >= tracks_->size()) { return QVariant(); }
  if (role == Qt::DisplayRole) {
    switch (index.column()) {
      case 0:
        return QString::number(tracks_->at(index.row()).firstFrameIndex())+":"
                    + QString::number(tracks_->at(index.row()).lastFrameIndex());
      case 1:  return QString::number(tracks_->at(index.row()).nodesCount());
      case 2:  return QVariant();
      default: return QVariant();
  } }
  else if (role == Qt::CheckStateRole) {
    switch (index.column()) {
      case 0:  return QVariant();
      case 1:  return QVariant();
      case 2:  return ((tracks_->at(index.row()).isRevised()) ? Qt::Checked
                                                              : Qt::Unchecked);
      default: return QVariant();
  } }
  else if (role == Qt::TextAlignmentRole) {
    switch (index.column()) {
      case 0:  return Qt::AlignRight;
      case 1:  return Qt::AlignLeft;
      case 2:  return Qt::AlignCenter;
      default: QVariant();
  } }
  return QVariant();
}


QVariant TracksModel::headerData(int section, Qt::Orientation orientation,
                                 int role) const
{
  if (orientation == Qt::Horizontal) {
    if (role == Qt::DisplayRole) {
      switch (section) {
        case 0:  return QString("Range");
        case 1:  return QString("Length");
        case 2:  return QString("OK"); // ☻☺
        default: return QVariant();
    } }
    else if (role == Qt::ToolTipRole) {
      switch (section) {
        case 2:  return QString("Revision status");
        default: return QVariant();
    } }
  } else {  //(vertical) base-1 counting
    if (role == Qt::DisplayRole) {
      return QString("%1").arg(section + 1);
  } }
  return QVariant();
}


Qt::ItemFlags TracksModel::flags(QModelIndex const& index) const
{
  if (!index.isValid()) { return Qt::ItemIsEnabled; }
  if (index.column() == 2) {
    return (QAbstractItemModel::flags(index) | Qt::ItemIsUserCheckable );
  } else {
    return  QAbstractItemModel::flags(index);
  }
}


bool TracksModel::setData( QModelIndex const& index, QVariant const& value,
                           int role )
{
  if (index.isValid() && (role == Qt::CheckStateRole)) {
    if (index.column() == 2) {
      assert(index.row() < tracks_->size());
      (*tracks_)[index.row()].setRevised(value.toBool());
      emit dataChanged(index,index);
      return true;
    }
  }
  return false;
}
