// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#ifndef SHUTTLETRACKER_GUI_SUPPORT_TRACKSMODEL_HPP
#define SHUTTLETRACKER_GUI_SUPPORT_TRACKSMODEL_HPP

#include "GUI/GUICommon.hpp"
#include "Track.hpp"
#include <QtGlobal>  // Q_DECL_OVERRIDE
#if !defined(HAS_QT5) && !defined(Q_DECL_OVERRIDE)
#  define Q_DECL_OVERRIDE
#endif
#include <QAbstractTableModel>

class TracksModel : public QAbstractTableModel
{
  Q_OBJECT

public:
  TracksModel(QList<Track>* tracks, int n_frames=0, QObject* parent=nullptr)
  : QAbstractTableModel{parent},
    tracks_{tracks},
    n_frames_{n_frames}
  { assert(tracks); }

  int rowCount(QModelIndex const& = QModelIndex()) const OVERRIDE;
  int columnCount(QModelIndex const& = QModelIndex()) const OVERRIDE;
  QVariant data(QModelIndex const& index, int role) const OVERRIDE;
  QVariant headerData(int section, Qt::Orientation orientation,
                      int role=Qt::DisplayRole) const OVERRIDE;

  // editability:
  Qt::ItemFlags flags(QModelIndex const& index) const OVERRIDE;
  bool setData(QModelIndex const& index, QVariant const& value,
               int role=Qt::EditRole) OVERRIDE;

protected:
  QList<Track>* tracks_ = nullptr;
  int const n_frames_ = 0;
};

#endif // SHUTTLETRACKER_GUI_SUPPORT_TRACKSMODEL_HPP
