// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#ifndef SHUTTLETRACKER_GUI_SUPPORT_ZOOMABLEQGRAPHICSVIEW_HPP
#define SHUTTLETRACKER_GUI_SUPPORT_ZOOMABLEQGRAPHICSVIEW_HPP

#include "GUI/GUICommon.hpp"
#include <QGraphicsView>
#include <QSet>

class ZoomableGraphicsView : public QGraphicsView
{
  Q_OBJECT

public:
//explicit ZoomableGraphicsView(QWidget* parent = 0);
  explicit ZoomableGraphicsView(QGraphicsScene* scene, QWidget* parent=nullptr);
  ~ZoomableGraphicsView() OVERRIDE;

  void addCoScrolling(ZoomableGraphicsView *pz) { co_scrolling_.push_back(pz); }

  void canInsert(QSet<int> const& s) { insertable_= s;  maybeChangePen_(); }
  void canErase (QSet<int> const& s) { erasable_  = s; }
  void canSwap  (QSet<int> const& s) { swappable_ = s; }
  void canToggle(QSet<int> const& s) { togglable_ = s; }
  void quantifies(bool whether) { quantifying_ = whether; }

signals:
  void itemAdded(QVector<QPoint> const&);
  void itemRemoved(QGraphicsItem const*);
  void itemToggled(QGraphicsItem const*);
  void itemReplaced(QGraphicsItem const*, QVector<QPoint> const&);
  void itemSplit(QGraphicsItem const*, QVector<QPoint> const&);
  void lineSelected(QGraphicsLineItem const*);
  void rectSelected(QGraphicsRectItem const*);
  void lineLineSelected(QGraphicsLineItem const*, QGraphicsLineItem const*);
  void lineRectSelected(QGraphicsLineItem const*, QGraphicsRectItem const*);
  void rectLineSelected(QGraphicsRectItem const*, QGraphicsLineItem const*);
  void rectRectSelected(QGraphicsRectItem const*, QGraphicsRectItem const*);
  void itemExtended(QGraphicsPolygonItem const*);
  void polygonToQuantSelected(QGraphicsPolygonItem const*);
  void polygonToExternalHighlight(QGraphicsPolygonItem const*);

public slots:

protected:
  void wheelEvent(QWheelEvent* event) OVERRIDE;
  void keyPressEvent(QKeyEvent* event) OVERRIDE;
  void mousePressEvent(QMouseEvent* event) OVERRIDE;
  void mouseReleaseEvent(QMouseEvent* event) OVERRIDE;
  void mouseDoubleClickEvent(QMouseEvent* event) OVERRIDE;
  void mouseMoveEvent(QMouseEvent* event) OVERRIDE;

  void drawContour_(bool close=false);
  void drawContourf_(bool close=false);
  void clearContourLines_();
  void maybeChangePen_();

  QVector<ZoomableGraphicsView*> co_scrolling_ = {};

  QGraphicsItem* selected_ = nullptr;
  QGraphicsItem* last_selected_ = nullptr;

  QVector<QPoint>  static contouri_; // for returning in item_added
  QVector<QPointF> static contourf_; // for drawing in the scene

  bool drawing_     = false,
       replacing_   = false,
       splitting_   = false,
       quantifying_ = false;

  QSet<int> insertable_ = {},  // integer-coded
            erasable_   = {},  // types of
            swappable_  = {},  // graphics
            togglable_  = {};  // items

  QVector<QGraphicsLineItem*> contour_lines_ = {};
  QPen contour_pen_ = QPen{QColor{Qt::blue}.lighter(133), 0.25,
                           Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin};

  qreal const scaling_factor_ = 0.9;
}; // class ZoomableGraphicsView

#endif // !defined(SHUTTLETRACKER_GUI_SUPPORT_ZOOMABLEQGRAPHICSVIEW_HPP)
