// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#include "AboutDialog.hpp"
#include "ui_AboutDialog.h"
#include <QRegularExpression>
#include <boost/config.hpp>


AboutDialog::AboutDialog(QWidget *parent)
: QDialog(parent),
  ui_(new Ui::AboutDialog)
{
  ui_->setupUi(this);

  // -- program name and version
  ui_->labelAppNameVersion->setText(QString("ShuttleTracker")
                                    + " <b>" + APP_VERSION + "</b>");

  // -- compiler version and date of build
  QStringList compiler_strs = QString(BOOST_COMPILER).split(" ");
  QString compiler = "";
  for (int i=0; i< compiler_strs.size(); ++i) {
    if (compiler_strs[i].contains("ersion")) { continue; }
    if (compiler_strs[i].contains("(")) { continue; }
    compiler += compiler_strs[i] + " ";
  }

  QRegularExpression static const double_space{"  "};
  QString const date = QString(__DATE__).replace(double_space, " ");
  ui_->labelCompiler->setText("Compiled with "+ compiler +"\non "+ date + ".");
  ui_->labelCompiler->setText( \
      QString(ui_->labelCompiler->text()).insert( \
          ui_->labelCompiler->text().length() - 6, ',' ) );

  // -- author and e-mail address
  auto emadr = "Pbagnpg: fuhggyrgenpxre.fbsgjner@tznvy.pbz";
  auto c13 = [](char c) -> char
  { if (     c >= 'a' && c <= 'z') { return (c - 'a' + 13) % 26 + 'a'; }
    else if (c >= 'A' && c <= 'Z') { return (c - 'A' + 13) % 26 + 'A'; }
    return c;
  };
  auto s13 = [c13](QString const& s) -> QString
  { QString rs;  for (auto &c: s.toStdString()) { rs += c13(c); }  return rs; };

  ui_->labelAppAuthor->setText( "Author: Marek Kochanczyk, IPPT PAN, Warsaw\n"
                                + s13(emadr) );
  //
  ui_->labelAppTesters->setText( "Testing, feedback, discussions: "
                                 "Maciej Dobrzynski"    ", "
                                 "Frederic Grabowski"   ", "
                                 "Piotr Korczyk"        ", "
                                 "Zbyszek Korwek"       ", "
                                 "Nont Kosaisawe"       ", "
                                 "Tomek Lipniacki"      ", "
                                 "Joasia Markiewicz"    ", "
                                 "Paweł Nałęcz-Jawecki" ", "
                                 "Wiktor Prus"          ", and "
                                 "Karolina Tudelska");

  // -- scripts directory
  ui_->labelScriptsDir->setText("Scripts directory: " + scripts_dir_path_);

  // -- hide some elements
  ui_->buttonBox->setVisible(false);
  ui_->lineScriptsDirBottom->setVisible(false);
}


AboutDialog::~AboutDialog()
{  delete ui_;  }


void AboutDialog::tellScriptsDirPath(const QString &dir_path)
{
  scripts_dir_path_ = dir_path;
  ui_->labelScriptsDir->setText("Scripts directory: " + scripts_dir_path_);
}
