// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#include "MainWindow_aux.hpp"
#include <QGraphicsScene>
#include <QGraphicsPixmapItem>

namespace aux {

bool isVersionMoreRecent(QString const& query_v, QString const& ref_v)
{
  QStringList const ref_v_s  =  ref_v.split('.'),
                  query_v_s = query_v.split('.');
  if (ref_v_s.size() != query_v_s.size()) { return false; }

  const int n_parts = ref_v_s.size();
  for (int part_i=0; part_i<n_parts; ++part_i) {
    if      (query_v_s[part_i].toInt() > ref_v_s[part_i].toInt()) {return true; }
    else if (query_v_s[part_i].toInt() < ref_v_s[part_i].toInt()) {return false;}
    else { continue; }
  }

  return false;
}


bool saveToPixmapFile(QGraphicsScene& scene, QString const& filename)
{
  QImage img(static_cast<int>(scene.width()), static_cast<int>(scene.height()),
             QImage::Format_ARGB32_Premultiplied);
  QPainter p(&img);
  p.setRenderHint(QPainter::Antialiasing);
  scene.render(&p);
  p.end();
  bool const success = img.save(filename);
  return success;
}


QVector<QPoint> trimmedToSceneBorder(QVector<QPoint>const& src, QSize const& sz)
{
  QVector<QPoint> dst;
  for (auto const& p: src) {
    if (    (p.x() >= 0) and (p.x() <= sz.width() )
        and (p.y() >= 0) and (p.y() <= sz.height())) {
      dst.push_back(p);
  } }
  return dst;
}

} // namespace aux
