// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#include "MainWindow.hpp"
#include "ui_MainWindow.h"
#include <iostream>  // std::clog

// slot
void MainWindow::message_(QChar cat, QString const& msg, int suggested_timeout)
{
  // QMutex static _;  QMutexLocker __(&_);
  if (msg.length() == 0) { return; }

  QString categ_msg;  // categorized message
  switch (cat.toLatin1()) {  // add one of custom prefixes:
    case 'I':  categ_msg += "Info: ";    break;
    case 'W':  categ_msg += "Warning: "; break;
    case 'E':  categ_msg += "Error: ";   break;
    case 'D':  categ_msg += "Devel: ";   break;
    default: break;
  }

  int timeout = (suggested_timeout >= 0) ? suggested_timeout : 0;
  if (suggested_timeout < 0) {
    switch (cat.toLatin1()) {  // add one of custom prefixes:
      case 'I':  timeout =  6000;  break;
      case 'W':  timeout =  6000;  break;
      case 'E':  timeout = 12000;  break;
      case 'D':  timeout = 12000;  break;
      default:   timeout =  6000;  break;
  } }

  if (batch_processing_) {
    categ_msg +=  "[Frames remaining: "
                 + QString::number(n_frames_ - current_frame_i_ - 1) + "] ";
  } else {
    //categ_msg += " [Frame " + QString::number(current_frame_i_) + "] ";
  }

  // Append the message body:
  categ_msg += msg;

  // Add full stop if missing:
  QChar const last_char = categ_msg.at(categ_msg.length()-1);
  if (    last_char != '.'
      and last_char != '!'
      and last_char != ')'
      and last_char != ']') { categ_msg += "."; }

  // Express three-fold:
  std::clog << categ_msg.toStdString();
  if (categ_msg.endsWith("...")) std::clog.flush(); else std::clog << std::endl;
  log_ << categ_msg;
  ui_->statusBar->showMessage(categ_msg, timeout);
}


// slot
void MainWindow::messageDone_()
{
  if (not log_.empty()) {
    QString const done_str = " done";
    log_.last() += done_str;
    std::clog << done_str.toStdString() << std::endl;
  }
  ui_->statusBar->showMessage("");  // reset
}
