// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#include "MainWindow.hpp"
#include "ui_MainWindow.h"
#include "Image/ImageOperations.hpp"
#include "GUI/Support/ZoomableGraphicsView.hpp"
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
#  include <QtConcurrent/QtConcurrent>
#  include <QtConcurrent/QtConcurrentRun>
#else
#  include <QtConcurrentRun>
#endif
#if (QT_VERSION >= QT_VERSION_CHECK(5, 2, 0))
#  ifdef Q_OS_WIN
#    include <QWinTaskbarButton>
#    include <QWinTaskbarProgress>
#  endif
#endif

// slot
void MainWindow::menuViewUpdate_()
{
  QString const sHide = "Hide";
  QString const sShow = "Show";

  bool const has_red_images   = images_->has(ImageChannel::R);
  bool const has_blue_images   = images_->has(ImageChannel::B);
  bool const has_green_images   = images_->has(ImageChannel::G);
  bool const has_cyan_images     = images_->has(ImageChannel::C);
  bool const has_magenta_images   = images_->has(ImageChannel::M);
  bool const has_yellow_images     = images_->has(ImageChannel::Y);
  bool const has_brightfield_images = images_->has(ImageChannel::BF);

  // -- RGB
  //
  ui_->menuGreenChannel->setEnabled(has_green_images);
  ui_->actionGreenChannelPanelToggle->setEnabled(has_green_images);
  ui_->actionGreenChannelPanelToggle->setText(has_green_images ? sHide : sShow);
  ui_->actionGreenChannelVisibilityToggle->setEnabled(has_green_images);
  if (has_green_images and images_->description(ImageChannel::G) != "G") {
    ui_->menuGreenChannel->setTitle(
        QString("Green channel: ") +
        QString::fromStdString(images_->description(ImageChannel::G)));
  } else {
    ui_->menuGreenChannel->setTitle("Green channel");
  }
  //
  ui_->menuRedChannel->setEnabled(has_red_images);
  ui_->actionRedChannelPanelToggle->setEnabled(has_red_images);
  ui_->actionRedChannelPanelToggle->setText(has_red_images ? sHide : sShow);
  ui_->actionRedChannelVisibilityToggle->setEnabled(has_red_images);
  if (has_red_images and images_->description(ImageChannel::R) != "R") {
    ui_->menuRedChannel->setTitle(
        QString("Red channel: ") +
        QString::fromStdString(images_->description(ImageChannel::R)));
  } else {
    ui_->menuRedChannel->setTitle("Red channel");
  }
  //
  ui_->menuBlueChannel->setEnabled(has_blue_images);
  ui_->actionBlueChannelPanelToggle->setEnabled(has_blue_images);
  ui_->actionBlueChannelPanelToggle->setText(has_blue_images ? sHide : sShow);
  ui_->actionBlueChannelVisibilityToggle->setEnabled(has_blue_images);
  if (has_blue_images and images_->description(ImageChannel::B) != "B") {
    ui_->menuBlueChannel->setTitle(
        QString("Blue channel: ") +
        QString::fromStdString(images_->description(ImageChannel::B)));
  } else {
    ui_->menuBlueChannel->setTitle("Blue channel");
  }

  // -- CMY
  //
  ui_->menuCyanChannel->setEnabled(has_cyan_images);
  ui_->actionCyanChannelPanelToggle->setEnabled(has_cyan_images);
  ui_->actionCyanChannelPanelToggle->setText(has_cyan_images ? sHide : sShow);
  ui_->actionCyanChannelVisibilityToggle->setEnabled(has_cyan_images);
  if (has_cyan_images and images_->description(ImageChannel::C) != "C") {
    ui_->menuCyanChannel->setTitle(
        QString("Cyan channel: ") +
        QString::fromStdString(images_->description(ImageChannel::C)));
  } else {
    ui_->menuCyanChannel->setTitle("Cyan channel");
  }
  //
  ui_->menuMagentaChannel->setEnabled(has_magenta_images);
  ui_->actionMagentaChannelPanelToggle->setEnabled(has_magenta_images);
  ui_->actionMagentaChannelPanelToggle->setText(has_magenta_images ? sHide : sShow);
  ui_->actionMagentaChannelVisibilityToggle->setEnabled(has_magenta_images);
  if (has_magenta_images and images_->description(ImageChannel::M) != "M") {
    ui_->menuMagentaChannel->setTitle(
        QString("Magenta channel: ") +
        QString::fromStdString(images_->description(ImageChannel::M)));
  } else {
    ui_->menuMagentaChannel->setTitle("Magenta channel");
  }
  //
  ui_->menuYellowChannel->setEnabled(has_yellow_images);
  ui_->actionYellowChannelPanelToggle->setEnabled(has_yellow_images);
  ui_->actionYellowChannelPanelToggle->setText(has_yellow_images ? sHide : sShow);
  ui_->actionYellowChannelVisibilityToggle->setEnabled(has_yellow_images);
  if (has_yellow_images and images_->description(ImageChannel::Y) != "Y") {
    ui_->menuYellowChannel->setTitle(
        QString("Yellow channel: ") +
        QString::fromStdString(images_->description(ImageChannel::Y)));
  } else {
    ui_->menuYellowChannel->setTitle("Yellow channel");
  }

  ui_->menuBrightfield->setEnabled(has_brightfield_images);
  ui_->actionBrightfieldPanelToggle->setEnabled(has_brightfield_images);
  ui_->actionBrightfieldPanelToggle->setText(has_brightfield_images ? sHide : sShow);
  ui_->actionBrightfieldVisibilityToggle->setEnabled(has_brightfield_images);

  bool const has_overlays = true;
  ui_->menuOverlays->setEnabled(has_overlays);
  ui_->actionOverlaysPanelToggle->setEnabled(has_overlays);
  ui_->actionOverlaysPanelToggle->setText(has_overlays ? sHide : sShow);
  ui_->actionOverlaysVisibilityToggle->setEnabled(has_overlays);
  //
  ui_->actionOverlaysGreenChannelInclusionToggle->setEnabled(has_green_images);
  ui_->actionOverlaysGreenChannelInclusionToggle->setText(has_green_images ?
      "Exclude green channel" : "Include green channel");
  ui_->actionOverlaysRedChannelInclusionToggle->setEnabled(has_red_images);
  ui_->actionOverlaysRedChannelInclusionToggle->setText(has_red_images ?
      "Exclude red channel" : "Include red channel");
  ui_->actionOverlaysBlueChannelInclusionToggle->setEnabled(has_blue_images);
  ui_->actionOverlaysBlueChannelInclusionToggle->setText(has_blue_images ?
      "Exclude blue channel" : "Include blue channel");
  //
  ui_->actionOverlaysCyanChannelInclusionToggle->setEnabled(has_cyan_images);
  ui_->actionOverlaysCyanChannelInclusionToggle->setText(has_cyan_images ?
      "Exclude cyan channel" : "Include cyan channel");
  ui_->actionOverlaysMagentaChannelInclusionToggle->setEnabled(has_magenta_images);
  ui_->actionOverlaysMagentaChannelInclusionToggle->setText(has_magenta_images ?
      "Exclude magenta channel" : "Include magenta channel");
  ui_->actionOverlaysYellowChannelInclusionToggle->setEnabled(has_yellow_images);
  ui_->actionOverlaysYellowChannelInclusionToggle->setText(has_yellow_images ?
      "Exclude yellow channel" : "Include yellow channel");
  //
  ui_->actionOverlaysBrightfieldInclusionToggle->setEnabled(has_brightfield_images);
  ui_->actionOverlaysBrightfieldInclusionToggle->setText(has_brightfield_images ?
      "Exclude bright-field" : "Include bright-field");

  menuViewEnable_(true);

  // TODO -- all actions below should be communicated, should suffice to toggle one
  ui_->actionShowOriginal->setChecked(true);
  ui_->actionShowOriginalDesaturated->setChecked(false);
  ui_->actionShowNormalized->setChecked(false);
  ui_->actionShowNormalizedDesaturated->setChecked(false);
  ui_->actionShowAutoleveledWeak->setChecked(false);
  ui_->actionShowAutoleveledWeakDesaturated->setChecked(false);
  ui_->actionShowAutoleveledMedium->setChecked(false);
  ui_->actionShowAutoleveledMediumGrayscale->setChecked(false);
  ui_->actionShowAutoleveledStrong->setChecked(false);
  ui_->actionShowAutoleveledStrongDesaturated->setChecked(false);

  // by default do not show overlays
  if (has_overlays) { ui_->actionOverlaysPanelToggle->trigger(); }

  // if it is the preference, do not show bright-field images
  if (ui_->actionAfterOpenHideBrightfield->isChecked()) {
    assert(has_brightfield_images == ui_->actionBrightfieldPanelToggle->isEnabled());
    if (ui_->actionBrightfieldPanelToggle->isEnabled()) {
      on_actionBrightfieldPanelToggle_triggered();
  } }
}


void MainWindow::menuViewEnable_(bool on)
{
  ui_->menuView->setEnabled(on);

  ui_->menuShowOriginal->setEnabled(on);
  ui_->actionShowOriginal->setEnabled(on);
  ui_->actionShowOriginalDesaturated->setEnabled(on);

  ui_->menuShowNormalized->setEnabled(on);
  ui_->actionShowNormalized->setEnabled(on);
  ui_->actionShowNormalizedDesaturated->setEnabled(on);

  ui_->menuShowAutoleveledWeak->setEnabled(on);
  ui_->actionShowAutoleveledWeak->setEnabled(on);
  ui_->actionShowAutoleveledWeakDesaturated->setEnabled(on);

  ui_->menuShowAutoleveledMedium->setEnabled(on);
  ui_->actionShowAutoleveledMedium->setEnabled(on);
  ui_->actionShowAutoleveledMediumGrayscale->setEnabled(on);

  ui_->menuShowAutoleveledStrong->setEnabled(on);
  ui_->actionShowAutoleveledStrong->setEnabled(on);
  ui_->actionShowAutoleveledStrongDesaturated->setEnabled(on);
}


ImageChannel MainWindow::channel(int panel_index)
{
  int panel_scan_index = 0;
  for (auto channel: st::AllChannels) {
    if (images_->has(channel)) {
      if (panel_scan_index == panel_index) { return channel; }
      ++panel_scan_index;
  } }
  return ImageChannel::UNDETERMINED;
}


// slots' aux funcs' aux func:
int MainWindow::panelIndex(ImageChannel target_channel)
{
  int panel_index = 0;
  for (auto channel: st::AllChannels) {
    if (channel == target_channel) {
      assert(images_->has(channel));
      return panel_index;
    } else if (images_->has(channel)) {
      ++panel_index;
    }
  }
  assert(target_channel == ImageChannel::UNDETERMINED);
  return panel_index;
}


// slots' aux func:
void MainWindow::togglePanel_(ImageChannel target_channel)
{
  QString const sHide = "Hide";
  QString const sShow = "Show";

  QAction* const a = *channel_toggle_.find(target_channel);  assert(a);
  int const panel_idx = panelIndex(target_channel);

  if (a->text() == sHide) {
    views_[panel_idx]->setVisible(false);
    a->setText(sShow);
  } else if (a->text() == sShow) {
    views_[panel_idx]->setVisible(true);
    a->setText(sHide);
  } else {
    assert(2+2 != 4);
    message_('E',  "[Internal]: Cannot toggle panel for channel "
                  + QString::fromStdString(st::toString(target_channel)));
  }
}
//
//
// slots:
void MainWindow::on_actionGreenChannelPanelToggle_triggered()
{  togglePanel_(ImageChannel::G);  }

void MainWindow::on_actionRedChannelPanelToggle_triggered()
{  togglePanel_(ImageChannel::R);  }

void MainWindow::on_actionBlueChannelPanelToggle_triggered()
{  togglePanel_(ImageChannel::B);  }

void MainWindow::on_actionCyanChannelPanelToggle_triggered()
{  togglePanel_(ImageChannel::C);  }

void MainWindow::on_actionMagentaChannelPanelToggle_triggered()
{ togglePanel_(ImageChannel::M);  }

void MainWindow::on_actionYellowChannelPanelToggle_triggered()
{  togglePanel_(ImageChannel::Y);  }

void MainWindow::on_actionBrightfieldPanelToggle_triggered()
{  togglePanel_(ImageChannel::BF); }

void MainWindow::on_actionOverlaysPanelToggle_triggered()
{ togglePanel_(ImageChannel::UNDETERMINED); }


// slots' aux func:
void MainWindow::toggleVisibility_(ImageChannel channel)
{
  // TODO initialize in another place
  QMap<ImageChannel,QAction*> kind2channel;
  kind2channel.insert(ImageChannel::G, ui_->actionGreenChannelVisibilityToggle);
  kind2channel.insert(ImageChannel::R, ui_->actionRedChannelVisibilityToggle);
  kind2channel.insert(ImageChannel::B, ui_->actionBlueChannelVisibilityToggle);
  kind2channel.insert(ImageChannel::C, ui_->actionCyanChannelVisibilityToggle);
  kind2channel.insert(ImageChannel::M, ui_->actionMagentaChannelVisibilityToggle);
  kind2channel.insert(ImageChannel::Y, ui_->actionYellowChannelVisibilityToggle);
  kind2channel.insert(ImageChannel::BF,ui_->actionBrightfieldVisibilityToggle);
  kind2channel.insert(ImageChannel::UNDETERMINED,ui_->actionOverlaysVisibilityToggle);

  QAction* const a = *kind2channel.find(channel);
  QString const static hide_str = "Black-out", show_str = "Display";

  if (a->text() == hide_str) {
    pixmap_cache_->blackout(channel);
    a->setText(show_str);
  } else if (a->text() == show_str) {
    pixmap_cache_->deblackout(channel);
    a->setText(hide_str);
  }

  int const panel_idx = panelIndex(channel);
  pixmapitems_[panel_idx]->setPixmap(
      pixmap_cache_->at(static_cast<int>(current_frame_i_), channel));
} // toggleVisibility
//
//
// slots:
void MainWindow::on_actionRedChannelVisibilityToggle_triggered()
{ toggleVisibility_(ImageChannel::R); }

void MainWindow::on_actionGreenChannelVisibilityToggle_triggered()
{ toggleVisibility_(ImageChannel::G); }

void MainWindow::on_actionBlueChannelVisibilityToggle_triggered()
{ toggleVisibility_(ImageChannel::B); }

void MainWindow::on_actionCyanChannelVisibilityToggle_triggered()
{ toggleVisibility_(ImageChannel::C); }

void MainWindow::on_actionMagentaChannelVisibilityToggle_triggered()
{ toggleVisibility_(ImageChannel::M); }

void MainWindow::on_actionYellowChannelVisibilityToggle_triggered()
{ toggleVisibility_(ImageChannel::Y); }

void MainWindow::on_actionBrightfieldVisibilityToggle_triggered()
{ toggleVisibility_(ImageChannel::BF); }

void MainWindow::on_actionOverlaysVisibilityToggle_triggered()
{ toggleVisibility_(ImageChannel::UNDETERMINED); }


// aux for slots below
void MainWindow::showActionsUncheckOthers_(QObject const* sender)
{
  assert(sender);
  for (QAction* pa: qAsConst(menu_view_show_actions_)) {
    if (pa != static_cast<QAction const*>(sender)) {
      pa->setChecked(false);
  } }
}

// aux for slots below
void MainWindow::on_actionShowAux(bool desaturated, ImageFilter filter)
{
  assert(filter);
  showActionsUncheckOthers_(QObject::sender());
  display_filter_ = filter;

  if (images_->hasOverlays()) { regenerateOverlays_(); }

  if (pixmap_cache_->hasCache()) {
    message_('I', "Caching converted images...");
    repaint();
  }
  pixmap_cache_->rebuild(current_frame_i_, desaturated);
  if (pixmap_cache_->hasCache()) {
    messageDone_();
  }

  showFrame_(current_frame_i_);
}

// slots:

void MainWindow::on_actionShowOriginal_triggered()
{ on_actionShowAux(false, st::identity); }

void MainWindow::on_actionShowOriginalDesaturated_triggered()
{ on_actionShowAux(true, st::identity); }

void MainWindow::on_actionShowNormalized_triggered()
{ on_actionShowAux(false, st::normalize); }

void MainWindow::on_actionShowNormalizedDesaturated_triggered()
{ on_actionShowAux(true, st::normalize); }

void MainWindow::on_actionShowAutoleveledWeak_triggered()
{ on_actionShowAux(false, st::autolevelWeak); }

void MainWindow::on_actionShowAutoleveledWeakDesaturated_triggered()
{ on_actionShowAux(true, st::autolevelWeak); }

void MainWindow::on_actionShowAutoleveledMedium_triggered()
{ on_actionShowAux(false, st::autolevelMedium); }

void MainWindow::on_actionShowAutoleveledMediumGrayscale_triggered()
{ on_actionShowAux(true, st::autolevelMedium); }

void MainWindow::on_actionShowAutoleveledStrong_triggered()
{ on_actionShowAux(false, st::autolevelStrong); }

void MainWindow::on_actionShowAutoleveledStrongDesaturated_triggered()
{ on_actionShowAux(true, st::autolevelStrong); }


void MainWindow::toggleOverlayComponent_(ImageChannel channel)
{
  // TODO initialize in another place
  QMap<ImageChannel,QAction*> channel2action;
  channel2action.insert(ImageChannel::G, ui_->actionOverlaysGreenChannelInclusionToggle);
  channel2action.insert(ImageChannel::R, ui_->actionOverlaysRedChannelInclusionToggle);
  channel2action.insert(ImageChannel::B, ui_->actionOverlaysBlueChannelInclusionToggle);
  channel2action.insert(ImageChannel::C, ui_->actionOverlaysCyanChannelInclusionToggle);
  channel2action.insert(ImageChannel::M, ui_->actionOverlaysMagentaChannelInclusionToggle);
  channel2action.insert(ImageChannel::Y, ui_->actionOverlaysYellowChannelInclusionToggle);
  channel2action.insert(ImageChannel::BF,ui_->actionOverlaysBrightfieldInclusionToggle);
  assert(channel2action.count(channel));

  QAction* const a = *channel2action.find(channel);
  std::set<ImageChannel>& excluded = images_->channelsExcludedFromOverlays();
  if (excluded.count(channel)) {
    excluded.erase(channel);
    a->setText(QString("Exclude ") + QString::fromStdString(toString(channel))
               + ((channel == ImageChannel::BF) ? "" : " channel"));
  } else {
    excluded.insert(channel);
    a->setText(QString("Include ") + QString::fromStdString(toString(channel))
               + ((channel == ImageChannel::BF) ? "" : " channel"));
  }
  if (images_->hasOverlays()) {
    regenerateOverlays_();
    pixmap_cache_->rebuild();
  } else {
    pixmap_cache_->rebuild(current_frame_i_);
  }

  pixmapitems_.back()->setPixmap(pixmap_cache_
                     ->last(static_cast<int>(current_frame_i_)));
}
//
//
// slot
void MainWindow::on_actionOverlaysRedChannelInclusionToggle_triggered()
{  toggleOverlayComponent_(ImageChannel::R);  }

void MainWindow::on_actionOverlaysGreenChannelInclusionToggle_triggered()
{  toggleOverlayComponent_(ImageChannel::G);  }

void MainWindow::on_actionOverlaysBlueChannelInclusionToggle_triggered()
{  toggleOverlayComponent_(ImageChannel::B);  }

void MainWindow::on_actionOverlaysCyanChannelInclusionToggle_triggered()
{  toggleOverlayComponent_(ImageChannel::C);  }

void MainWindow::on_actionOverlaysMagentaChannelInclusionToggle_triggered()
{  toggleOverlayComponent_(ImageChannel::M);  }

void MainWindow::on_actionOverlaysYellowChannelInclusionToggle_triggered()
{  toggleOverlayComponent_(ImageChannel::Y);  }

void MainWindow::on_actionOverlaysBrightfieldInclusionToggle_triggered()
{  toggleOverlayComponent_(ImageChannel::BF); }

void MainWindow::on_actionHideMarking_triggered(bool checked)
{
  for (auto& scene: scenes_) {
    auto const& items = scene->items();
    for (auto it: items) {
      if (   it->type() == QGraphicsPolygonItem::Type    // mask, nuc, reg
          or it->type() == QGraphicsLineItem::Type       // track
          or it->type() == QGraphicsTextItem::Type   ) { // nuc or reg label
        (not checked) ? it->show() : it->hide();
      }
  } }


  for (auto& tb: {ui_->toolBoxTool0, ui_->toolBoxTool1,
                  ui_->toolBoxTool2, ui_->toolBoxTool3,
                  ui_->toolBoxTool4, ui_->toolBoxTool5,
                  ui_->toolBoxTool6, ui_->toolBoxTool7}) {
    tb->setEnabled(not checked);
  }
}

void MainWindow::on_actionHideImages_triggered(bool /*checked*/)
{
  if (images_->has(ImageChannel::G))
  { on_actionGreenChannelVisibilityToggle_triggered(); }

  if (images_->has(ImageChannel::R))
  { on_actionRedChannelVisibilityToggle_triggered();   }

  if (images_->has(ImageChannel::B))
  { on_actionBlueChannelVisibilityToggle_triggered();  }

  if (images_->has(ImageChannel::C))
  { on_actionCyanChannelVisibilityToggle_triggered();  }

  if (images_->has(ImageChannel::M))
  { on_actionMagentaChannelVisibilityToggle_triggered();}

  if (images_->has(ImageChannel::Y))
  { on_actionYellowChannelVisibilityToggle_triggered();}

  if (images_->has(ImageChannel::BF))
  { on_actionBrightfieldVisibilityToggle_triggered();  }

  on_actionOverlaysVisibilityToggle_triggered();
}


void MainWindow::regenerateOverlays_()
{
  assert(images_->hasOverlays());
  message_('I', "Regenerating image overlays...");
  future_void_ = QtConcurrent::run(images_.get(),
                                   &ImageSequence::regenerateOverlays,
                                   display_filter_);
  repaint();
  future_void_.waitForFinished();
  messageDone_();
}
