// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#include "MainWindow.hpp"
#include "ui_MainWindow.h"
#include <QMainWindow>
#include <QMessageBox>
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
#  include <QThread>
#endif
#include <cassert>

namespace {
auto const sParamNucDetStainingNuclear   = "nuclei_detection_staining_nuclear";
auto const sParamNucDetChannel           = "nuclei_detection_channel";
auto const sParamNucDetNormalization     = "nuclei_detection_normalization";
auto const sParamNucDetDenoising         = "nuclei_detection_denoising";
auto const sParamNucDetDenoisingStrength = "nuclei_detection_denoising_strength";
auto const sParamNucDetSmoothing         = "nuclei_detection_smoothing";
auto const sParamNucDetSmoothingRadius   = "nuclei_detection_smoothing_radius";
auto const sParamNucDetThresholding                  = "nuclei_detection_thresholding";
auto const sParamNucDetThresholdingBaseline          = "nuclei_detection_thresholding_baseline_offset";
auto const sParamNucDetThresholdingBaselineOBSOLETE  = "nuclei_detection_thresholding_baseline";
auto const sParamNucDetThresholdingBlocksize         = "nuclei_detection_thresholding_block_size";
auto const sParamNucDetThresholdingBlocksizeOBSOLETE = "nuclei_detection_thresholding_blocksize";
auto const sParamNucDetThresholdingAuto              = "nuclei_detection_thresholding_auto";
auto const sParamNucDetThresholdingBaselineAutoFrom  = "nuclei_detection_thresholding_baseline_offset_auto_from";
auto const sParamNucDetThresholdingBaselineAutoUpto  = "nuclei_detection_thresholding_baseline_offset_auto_upto";
auto const sParamNucDetThresholdingBaselineAutoStep  = "nuclei_detection_thresholding_baseline_offset_auto_step";
auto const sParamNucDetThresholdingBlocksizeAutoFrom = "nuclei_detection_thresholding_block_size_auto_from";
auto const sParamNucDetThresholdingBlocksizeAutoUpto = "nuclei_detection_thresholding_block_size_auto_upto";
auto const sParamNucDetThresholdingBlocksizeAutoNSteps="nuclei_detection_thresholding_block_size_auto_steps";
auto const sParamNucDetUniformization          = "nuclei_detection_uniformization";
auto const sParamNucDetUniformizationThresholdOBSOLETE = "nuclei_detection_uniformization_threshold";
auto const sParamNucDetUniformizationThreshold = "nuclei_detection_uniformization_brightness_threshold";
auto const sParamNucDetOpening                 = "nuclei_detection_opening";
auto const sParamNucDetOpeningRepeats          = "nuclei_detection_opening_repeats";
auto const sParamNucDetOpeningErosionSize      = "nuclei_detection_opening_erosion_base_size";
auto const sParamNucDetOpeningDilationSize     = "nuclei_detection_opening_dilation_base_size";
auto const sParamNucDetOpeningErosionSizeOBSOLETE  = "nuclei_detection_opening_erosion_basesize";
auto const sParamNucDetOpeningDilationSizeOBSOLETE = "nuclei_detection_opening_dilation_base_ize";
auto const sParamNucDetClosing                 = "nuclei_detection_closing";
auto const sParamNucDetClosingRepeats          = "nuclei_detection_closing_repeats";
auto const sParamNucDetClosingDilationSize     = "nuclei_detection_closing_dilation_base_size";
auto const sParamNucDetClosingErosionSize      = "nuclei_detection_closing_erosion_base_size";
auto const sParamNucDetClosingDilationSizeOBSOLETE = "nuclei_detection_closing_dilation_base_size";
auto const sParamNucDetClosingErosionSizeOBSOLETE  = "nuclei_detection_closing_erosion_base_size";

auto const sParamNucDetContoursThresholdLowOBSOLETE  = "nuclei_detection_contour_detection_threshold_low";
auto const sParamNucDetContoursThresholdsRatioOBSOLETE="nuclei_detection_contour_detection_thresholds_ratio";
auto const sParamNucDetContoursToleranceOBSOLETE     = "nuclei_detection_contour_detection_tolerance";
auto const sParamNucDetContoursReconstructOBSOLETE   = "nuclei_detection_contour_detection_reconstruct";
auto const sParamNucDetContoursThresholdLow  = "nuclei_detection_contour_detection_threshold_low";
auto const sParamNucDetContoursThresholdsRatio="nuclei_detection_contour_detection_thresholds_ratio";
auto const sParamNucDetContoursTolerance     = "nuclei_detection_contour_detection_tolerance";
auto const sParamNucDetContoursReconstruct   = "nuclei_detection_contour_detection_reconstruct";

auto const sParamNucDetAssessmentMinSolidityOBSOLETE = "nuclei_assessment_min_solidity";
auto const sParamNucDetAssessmentAreaFracMinOBSOLETE = "nuclei_assessment_min_area";
auto const sParamNucDetAssessmentAreaFracMaxOBSOLETE = "nuclei_assessment_max_area";
auto const sParamNucDetSplittingOBSOLETE             = "nuclei_splitting";
auto const sParamNucDetSplittingMinPocketAreaOBSOLETE= "nuclei_splitting_min_indent";
auto const sParamNucDetAssessmentMinSolidity = "nuclei_detection_nuclei_assessment_min_solidity";
auto const sParamNucDetAssessmentAreaFracMin = "nuclei_detection_nuclei_assessment_min_area_as_median_fraction";
auto const sParamNucDetAssessmentAreaFracMax = "nuclei_detection_nuclei_assessment_max_area_as_median_fraction";
auto const sParamNucDetSplitting             = "nuclei_detection_nuclei_splitting_adjacent";
auto const sParamNucDetSplittingMinPocketArea= "nuclei_detection_nuclei_splitting_min_pocket_area";

auto const sParamNucEditNuccutChannelMain       = "nuclei_editing_nuccut_channel_main";
auto const sParamNucEditNuccutChannelAux1       = "nuclei_editing_nuccut_channel_aux1";
auto const sParamNucEditNuccutChannelAux2       = "nuclei_editing_nuccut_channel_aux2";
auto const sParamNucEditNuccutChannelMainWeight = "nuclei_editing_nuccut_channel_main_weight";
auto const sParamNucEditNuccutChannelAux1Weight = "nuclei_editing_nuccut_channel_aux1_weight";
auto const sParamNucEditNuccutChannelAux2Weight = "nuclei_editing_nuccut_channel_aux2_weight";
auto const sParamNucEditNuccutColorSimilarity   = "nuclei_editing_nuccut_color_conservativeness";
auto const sParamNucEditNuccutEdgeImpassability = "nuclei_editing_nuccut_edge_impassability";
auto const sParamNucEditNuccutExcludeToggled    = "nuclei_editing_nuccut_exclude_toggled";
auto const sParamNucEditShowingNucNumbers       = "nuclei_editing_nuclei_labels";

auto const sParamPerinucInnerOffset      = "perinuclei_inner_offset";
auto const sParamPerinucWidth            = "perinuclei_width";
auto const sParamPerinucNeighAvoidance   = "perinuclei_neighbor_avoidance";
auto const sParamPerinucOverlapAvoidance = "perinuclei_overlap_avoidance";
auto const sParamPerinucMinPartArea      = "perinuclei_min_part_area";
auto const sParamPerinucBgAvoid          = "perinuclei_background_avoidance";
auto const sParamPerinucBgAvoidChannel   = "perinuclei_background_avoidance_channel";
auto const sParamPerinucBgAvoidExpansion = "perinuclei_background_avoidance_expansion";
auto const sParamPerinucBgAvoidSmoothing = "perinuclei_background_avoidance_smoothing";

auto const sParamRgnEditBackgroundSubdivs  = "regions_background_subdivs";
auto const sParamRgnEditBackgroundCount    = "regions_background_count";
auto const sParamRgnEditBackgroundMergeAdj = "regions_background_merge_adjacent_tiles";

auto const sParamRgnEditVoronoiScaling      = "regions_voronoi_scaling";
auto const sParamRgnEditVoronoiErosion      = "regions_voronoi_erosion";
auto const sParamRgnEditVoronoiSavingNeighs ="regions_voronoi_exporting_neighborhoods";
auto const sParamRgnEditShowingRegNumbers   = "regions_labels";

auto const sParamQuantChannelG = "quantify_channel_green";
auto const sParamQuantChannelR = "quantify_channel_red";
auto const sParamQuantChannelB = "quantify_channel_blue";
auto const sParamQuantChannelC = "quantify_channel_cyan";
auto const sParamQuantChannelM = "quantify_channel_magenta";
auto const sParamQuantChannelY = "quantify_channel_yellow";

auto const sParamQuantChannelDescG = "channel_green_description";
auto const sParamQuantChannelDescR = "channel_red_description";
auto const sParamQuantChannelDescB = "channel_blue_description";
auto const sParamQuantChannelDescC = "channel_cyan_description";
auto const sParamQuantChannelDescM = "channel_magenta_description";
auto const sParamQuantChannelDescY = "channel_yellow_description";

auto const sParamQuantImgExportDat     = "quantify_image_to_csv_file";
auto const sParamQuantImgOnlyNonmasked = "quantify_image_only_nonmasked";
auto const sParamQuantNucExportDat     = "quantify_nuclei_to_csv_file";
auto const sParamQuantNucExportPng     = "quantify_nuclei_to_png_file";
auto const sParamQuantPerExportDat     = "quantify_perinuclei_to_csv_file";
auto const sParamQuantPerExportHalos   = "quantify_halos_to_csv_file";
auto const sParamQuantRgnExportDat     = "quantify_regions_to_csv_file";
auto const sParamQuantRgnExportPng     = "quantify_regions_to_png_file";

auto const sParamTrackingNucPosPrediction             = "tracking_prediction";
auto const sParamTrackingNucPosPredictionMemory       = "tracking_prediction_memory";
auto const sParamTrackingNucPosPredictionConservation = "tracking_prediction_conservation";
auto const sParamTrackingNucPosPredictionContribution = "tracking_prediction_contribution";
auto const sParamTrackingNucSimWeightProximity        = "tracking_similarity_weight_proximity";
auto const sParamTrackingNucSimWeightArea             = "tracking_similarity_weight_area";
auto const sParamTrackingNucSimWeightEccentricity     = "tracking_similarity_weight_eccentricity";
auto const sParamTrackingNucSimWeightOrientation      = "tracking_similarity_weight_orientation";
auto const sParamTrackingNucSimWeightIntensitySum     = "tracking_similarity_weight_intensity_sum";
auto const sParamTrackingNucSimWeightIntensityInertia = "tracking_similarity_weight_intensity_inertia";
auto const sParamTrackingNucSimWeightIntensityDistrib = "tracking_similarity_weight_intensity_distribution";

auto const sParamTrackingNucFeatureDistScaling = "tracking_feature_distance_scaling";
auto const sParamTrackingNucDisplacementCutoff = "tracking_displacement_cutoff";

auto const sParamTrackingNucAreaDrop          = "tracking_nucleus_area_drop";
auto const sParamTrackingNucAreaDropThreshold = "tracking_nucleus_area_drop_threshold";

auto const sParamTracksEditSaveOnlyRevised = "tracks_save_only_revised";
}


void MainWindow::updateScripts(QStringList scripts_paths)
{
  assert(script_window_.get());

  // clear
  auto const& actions = ui_->menuScript->actions();
  for (QAction* pa: actions) {
    if (pa->text() != "Edit...") {
      disconnect(pa, SIGNAL(triggered()),
                 this, SLOT(executeScriptFileByName()));
      delete pa;
  } }

  // fill
  unsigned int key_i = 0u;
  for (QString const& scr_path: scripts_paths) {
    QString scr_name = scr_path;
    scr_name.replace(script_window_->scriptFileNameExtension(), "");
    auto a = new QAction(scr_name, this);
    if (key_i < 10) {
      a->setShortcut(Qt::CTRL | Qt::SHIFT | (Qt::Key_0 + key_i++));
    }
    connect(a, SIGNAL(triggered()), this, SLOT(executeScriptFileByName()));
    ui_->menuScript->addAction(a);
  }
}


// slot
void MainWindow::executeScriptFileByName()
{
  if (not script_window_) { setupScripting_(); }
  QString const script_name = dynamic_cast<QAction*>(QObject::sender())->text();

  // show script window on execution failure
  if (script_window_->loadAndExecute(script_name)) {
    script_window_->raise();
    script_window_->show();
  }
}


// This workaround results from atricious MSCV2013 initz-list handling.
#ifdef _MSC_VER
QMap<QString,QString> MainWindow::api_functions; // empty here, filled in c'tor
const QVector<QPair<QString,QString>> MainWindow::api_functions__( {
#else
const QMap<QString,QString> MainWindow::api_functions( {
#endif
    {"open_directory","  open_directory($);"},
    {"detect_nuclei","  detect_nuclei();"},
    {"detect_nuclei_with_parameter_scan",
         "  detect_nuclei_with_parameter_scan();"},
    {"nuclei_count", "# nuclei_count();" },
    {"regions_count", "# regions_count();" },
    {"derive_perinuclei","  derive_perinuclei();"},
    {"sample_background","  sample_background();"},
    {"tesselate","  tesselate();"},
    {"load_nuclei_detection_settings","  load_nuclei_detection_settings();"},
    {"save_nuclei_detection_settings","  save_nuclei_detection_settings();"},
    {"load_perinuclei_settings","  load_perinuclei_settings();"},
    {"save_perinuclei_settings","  save_perinuclei_settings();"},
    {"remove_nuclei","  remove_nuclei();"},
    {"___remove_nucleus___", ""          }, // not visible in API listing
    {"remove_debris","  remove_debris();"},
    {"remove_border_nuclei","  remove_border_nuclei();"},
    {"remove_split_orphans", "  remove_split_orphans();"},
    {"remove_interior_debris", "  remove_interior_debris();"},
    {"remove_perinuclei","  remove_perinuclei();"},
    {"remove_regions","  remove_regions();"},
    {"remove_tracks","  remove_tracks();"},
    {"remove_tracks_nonrevised","  remove_tracks_nonrevised();"},
    {"toggle_all_nuclei", "  toggle_all_nuclei();"},
    {"toggle_border_nuclei", "  toggle_border_nuclei();"},
    {"track_nuclei","  track_nuclei();"  },
    {"has_masking_contours_file", "% has_masking_contours_file();"},
    {"has_nuclei_contours_file",  "% has_nuclei_contours_file();" },
    {"has_regions_contours_file", "% has_regions_contours_file();"},
    {"has_nuclei_detection_settings_file",
         "% has_nuclei_detection_settings_file();"},
    {"has_perinuclei_settings_file", "% has_perinuclei_settings_file();"},
    {"has_tracking_settings_file",   "% has_tracking_settings_file();"},
    {"load_masking_contours","# load_masking_contours();"  },
    {"load_nuclei_contours", "# load_nuclei_contours();"   },
    {"load_regions_contours","# load_regions_contours();"  },
    {"save_masking_contours","# save_masking_contours();"  },
    {"save_nuclei_contours", "# save_nuclei_contours();"   },
    {"save_regions_contours","# save_regions_contours();"  },
    {"apply_masking_contours","  apply_masking_contours();"},
    {"highlight_nuclei_from_file", "  highlight_nuclei_from_file();" },
    {"export_all_quantifications","  export_all_quantifications();"},
    {"export_nuclei_quantifications","  export_nuclei_quantifications();"},
    {"export_perinuclei_quantifications",
        "  export_perinuclei_quantifications();"},
    {"export_regions_quantifications","  export_regions_quantifications();"},
    {"export_image_quantifications","  export_image_quantifications();"},
    {"rename_per_qfile_as_far_qfile",  "  rename_per_qfile_as_far_qfile();"},
    {"load_tracks",  "# load_tracks();"  },
    {"save_tracks",  "# save_tracks();"  },
    {"save_tracking_settings", "# save_tracking_settings();"  },
    {"load_tracking_settings", "# load_tracking_settings();"  },
    {"toggle_overlay_channel", "  toggle_overlay_channel(*);" },
    {"frames_count", "# frames_count();" },
    {"tracks_count", "# tracks_count();" },
    {"current_frame","# current_frame();"},
    {"go_to_frame",  "  go_to_frame(#);" },
    {"set",          "  set($, *);"      },
    {"get",          "$ get($);"         },
    {"echo",         "  echo(*);"        },
    {"repaint",      "  repaint();"      },
    {"millisleep",   "  millisleep(#);"  },
    {"panic",        "  panic($);"       },
    {"quit",         "  quit();"         },
    {"___batch_processing___", ""        }  // not visible in API listing
} );


// This workaround results from atricious MSCV2013 initz-list handling.
#ifdef _MSC_VER
QSet<QString> MainWindow::api_parameters; // empty here, filled in c'tor
const QVector<QString> MainWindow::api_parameters__( {
#else
const QSet<QString> MainWindow::api_parameters( {
#endif
  sParamNucDetStainingNuclear,
  sParamNucDetChannel,
  sParamNucDetNormalization,
  sParamNucDetDenoising,
  sParamNucDetDenoisingStrength,
  sParamNucDetSmoothing,
  sParamNucDetSmoothingRadius,
  sParamNucDetThresholding,
  sParamNucDetThresholdingBaseline,
  sParamNucDetThresholdingBlocksize,
  sParamNucDetThresholdingAuto,
  sParamNucDetThresholdingBaselineAutoFrom,
  sParamNucDetThresholdingBaselineAutoUpto,
  sParamNucDetThresholdingBaselineAutoStep,
  sParamNucDetThresholdingBlocksizeAutoFrom,
  sParamNucDetThresholdingBlocksizeAutoUpto,
  sParamNucDetThresholdingBlocksizeAutoNSteps,
  sParamNucDetUniformization,
  sParamNucDetUniformizationThreshold,
  sParamNucDetOpening,
  sParamNucDetOpeningRepeats,
  sParamNucDetOpeningErosionSize,
  sParamNucDetOpeningDilationSize,
  sParamNucDetClosing,
  sParamNucDetClosingRepeats,
  sParamNucDetClosingDilationSize,
  sParamNucDetClosingErosionSize,
  sParamNucDetContoursThresholdLow,
  sParamNucDetContoursThresholdsRatio,
  sParamNucDetContoursTolerance,
  sParamNucDetContoursReconstruct,
  sParamNucDetAssessmentMinSolidity,
  sParamNucDetAssessmentAreaFracMin,
  sParamNucDetAssessmentAreaFracMax,
  sParamNucDetSplitting,
  sParamNucDetSplittingMinPocketArea,
  //
  sParamNucEditNuccutChannelMain,
  sParamNucEditNuccutChannelAux1,
  sParamNucEditNuccutChannelAux2,
  sParamNucEditNuccutChannelMainWeight,
  sParamNucEditNuccutChannelAux1Weight,
  sParamNucEditNuccutChannelAux2Weight,
  sParamNucEditNuccutColorSimilarity,
  sParamNucEditNuccutEdgeImpassability,
  sParamNucEditNuccutExcludeToggled,
  //
  sParamNucEditShowingNucNumbers,
  //
  sParamPerinucInnerOffset,
  sParamPerinucWidth,
  sParamPerinucNeighAvoidance,
  sParamPerinucOverlapAvoidance,
  sParamPerinucMinPartArea,
  sParamPerinucBgAvoid,
  sParamPerinucBgAvoidChannel,
  sParamPerinucBgAvoidExpansion,
  sParamPerinucBgAvoidSmoothing,
  //
  sParamRgnEditBackgroundSubdivs,
  sParamRgnEditBackgroundCount,
  sParamRgnEditBackgroundMergeAdj,
  sParamRgnEditVoronoiScaling,
  sParamRgnEditVoronoiErosion,
  sParamRgnEditVoronoiSavingNeighs,
  sParamRgnEditShowingRegNumbers,
  //
  sParamQuantChannelG,
  sParamQuantChannelR,
  sParamQuantChannelB,
  sParamQuantChannelC,
  sParamQuantChannelM,
  sParamQuantChannelY,
  sParamQuantChannelDescG,
  sParamQuantChannelDescR,
  sParamQuantChannelDescB,
  sParamQuantChannelDescC,
  sParamQuantChannelDescM,
  sParamQuantChannelDescY,
  //
  sParamQuantImgExportDat,
  sParamQuantImgOnlyNonmasked,
  sParamQuantNucExportDat,
  sParamQuantNucExportPng,
  sParamQuantPerExportDat,
  sParamQuantPerExportHalos,
  sParamQuantRgnExportDat,
  sParamQuantRgnExportPng,
  //
  sParamTrackingNucPosPrediction,
  sParamTrackingNucPosPredictionMemory,
  sParamTrackingNucPosPredictionConservation,
  sParamTrackingNucPosPredictionContribution,
  sParamTrackingNucSimWeightProximity,
  sParamTrackingNucSimWeightArea,
  sParamTrackingNucSimWeightEccentricity,
  sParamTrackingNucSimWeightOrientation,
  sParamTrackingNucSimWeightIntensitySum,
  sParamTrackingNucSimWeightIntensityInertia,
  sParamTrackingNucSimWeightIntensityDistrib,
  sParamTrackingNucFeatureDistScaling,
  sParamTrackingNucDisplacementCutoff,
  sParamTrackingNucAreaDrop,
  sParamTrackingNucAreaDropThreshold,
  //
  sParamTracksEditSaveOnlyRevised
} );


// This workaround results from atricious MSCV2013 initz-list handling.
#ifdef _MSC_VER
QSet<QString> MainWindow::api_specials; // empty here, filled in c'tor
const QVector<QString> MainWindow::api_specials__( {
#else
const QSet<QString> MainWindow::api_specials( {
#endif
  "blue", "green", "red", "cyan", "magenta", "yellow",
  "on", "off",
  "true", "false"
} );


void MainWindow::open_directory(QString d)
{
  loadImagesFromDirectory(d);

  // On windows, this is necessary to refresh taskbar button progress
  // (however insufficient to refresh the window...)
  do {
    millisleep(100);
    repaint();
  } while (future_int_.isRunning());
  future_int_.waitForFinished();
}

void MainWindow::detect_nuclei()
{  on_pushButtonNucDetDetect_clicked();  }

void MainWindow::detect_nuclei_with_parameter_scan()
{
  ui_->radioButtonNucDetParamsAuto->setChecked(true);
  on_pushButtonNucDetDetect_clicked();

  // On windows, this is necessary to refresh taskbar button progress
  // (however insufficient to refresh the window...)
  do {
    millisleep(100);
    repaint();
  } while (future_int_.isRunning());
  future_int_.waitForFinished();
}

void MainWindow::derive_perinuclei()
{  on_pushButtonPerinucCreate_clicked();  }

void MainWindow::sample_background()
{  on_pushButtonRgnEditBackgroundSample_clicked();  }

void MainWindow::tesselate()
{  on_pushButtonRgnEditTesselate_clicked();  }

void MainWindow::remove_toggled_nuclei()
{  on_pushButtonNucEditClearToggled_clicked();  }

void MainWindow::remove_nuclei()
{  on_pushButtonNucEditClearAll_clicked();  }

void MainWindow::___remove_nucleus___(int nuc_idx)
{
  assert(nuc_idx < current_nuclei_->size());
  current_nuclei_->removeAt(nuc_idx);
  if (ui_->checkBoxNucEditShowAnnotations->isChecked()) {
    clearFrameAnnotations_(true, false);
    makeFrameAnnotations_(true, false);
  }
}

void MainWindow::remove_split_orphans()
{  on_pushButtonNucEditRemoveSplitOrphans_clicked();  }

void MainWindow::remove_debris()
{  on_pushButtonNucEditRemoveDebris_clicked();  }

void MainWindow::remove_interior_debris()
{  on_pushButtonNucEditExcludeInteriorDebris_clicked();  }

void MainWindow::toggle_all_nuclei()
{  on_pushButtonNucEditToggleAll_clicked();  }

void MainWindow::toggle_border_nuclei()
{  on_pushButtonNucEditSwitchOnBorders_clicked();  }

void MainWindow::remove_border_nuclei()
{  on_pushButtonNucEditExcludeOnBorders_clicked();  }

void MainWindow::remove_perinuclei()
{  on_pushButtonPerinucClearAll_clicked();  }

void MainWindow::track_nuclei()
{  on_pushButtonTrackingStep_clicked();  }

bool MainWindow::has_masking_contours_file()
{  return checkIfImageMaskContoursFileAvailable_();  }
//
int MainWindow::load_masking_contours()
{  on_pushButtonImgMaskLoad_clicked();  return current_exclusions_.size();  }
//
int MainWindow::save_masking_contours()
{  on_pushButtonImgMaskSave_clicked();  return current_exclusions_.size();  }

bool MainWindow::has_nuclei_contours_file()
{  return checkIfNucleiContoursFileAvailable_();  }
//
int MainWindow::load_nuclei_contours()
{on_pushButtonNucEditNucleiContoursLoad_clicked();return current_nuclei_->size();}
//
int MainWindow::save_nuclei_contours()
{on_pushButtonNucEditNucleiContoursSave_clicked();return current_nuclei_->size();}

bool MainWindow::has_nuclei_detection_settings_file()
{  return checkIfNucDetSettingsFileAvailable_();  }
//
void MainWindow::load_nuclei_detection_settings()
{  on_pushButtonNucDetSettingsLoad_clicked();  }
//
void MainWindow::save_nuclei_detection_settings()
{  on_pushButtonNucDetSettingsSave_clicked();  }

bool MainWindow::has_perinuclei_settings_file()
{  return checkIfPerinucDerivSettingsFileAvailable_();  }
//
void MainWindow::load_perinuclei_settings()
{  on_pushButtonPerinucSettingsLoad_clicked();  }
//
void MainWindow::save_perinuclei_settings()
{  on_pushButtonPerinucSettingsSave_clicked();  }

bool MainWindow::has_tracking_settings_file()
{  return checkIfTrackingSettingsFileAvailable_();  }
//
void MainWindow::load_tracking_settings()
{  on_pushButtonTrackingSettingsLoad_clicked();  }
//
void MainWindow::save_tracking_settings()
{  on_pushButtonTrackingSettingsSave_clicked();  }

void MainWindow::apply_masking_contours()
{  on_pushButtonImgMaskApply_clicked();  }

void MainWindow::remove_regions()
{  on_pushButtonRgnEditReset_clicked();  }

bool MainWindow::has_regions_contours_file()
{  return checkIfRegionsContoursFileAvailable_();  }
//
int MainWindow::load_regions_contours()
{  on_pushButtonRgnEditRegionsLoad_clicked(); return current_regions_->size(); }
//
int MainWindow::save_regions_contours()
{  on_pushButtonRgnEditRegionsSave_clicked(); return current_regions_->size(); }

void MainWindow::toggle_overlay_channel(QString channel)
{
  bool found = false;
  for (auto const& ch: st::AllChannels) {
    if (images_->has(ch) and (   images_->description(ch) == channel.toStdString()
                              or st::toString(ch) == channel.toLower().toStdString())) {
      toggleOverlayComponent_(ch);
      found = true;
      break;
  } }
  if (not found) { panic(QString("Could not find channel ") + "'" + channel + "'"
                         " to toggle its image in overlay(s)."
                         " (Neither pseudo-color nor channel description matches.)"); }
}

void MainWindow::highlight_nuclei_from_file()
{
//ui_->actionShowAutoleveledWeak->trigger();
//ui_->actionShowOriginal->trigger();
  on_actionShowAux(/*desaturated=*/false, st::brighten);
  on_pushButtonNucEditMarkFromFile_clicked();
}

void MainWindow::export_all_quantifications()
{  on_pushButtonQuantSaveAll_clicked();  }
//
void MainWindow::export_nuclei_quantifications()
{  on_pushButtonNucQuantSaveAll_clicked();  }
//
void MainWindow::export_perinuclei_quantifications()
{  on_pushButtonQuantPerinucSaveAll_clicked();  }
//
void MainWindow::export_regions_quantifications()
{  on_pushButtonQuantRgnSaveAll_clicked();  }
//
void MainWindow::export_image_quantifications()
{  on_pushButtonQuantImgSaveAll_clicked();  }
//
void MainWindow::rename_per_qfile_as_far_qfile()
{
  QString const fn_per = assembleQuantsFilename_("per", "CSV");
  if (not QFileInfo::exists(fn_per)) {
    message_('W', "Perinuclei quantification file does not exist.");
    return;
  }

  QString const fn_far = assembleQuantsFilename_("far", "CSV");
  bool const ok = QFile::rename(fn_per, fn_far);
  if (not ok) {
    message_('E', "Could not rename the perinuclei quantification file to the "
                  "far quantification file.");
  }
}

int MainWindow::load_tracks()
{  on_pushButtonTracksEditLoad_clicked();  return tracks_.size();  }
//
int MainWindow::save_tracks()
{  on_pushButtonTracksEditSave_clicked();  return tracks_.size();  }

void MainWindow::remove_tracks()
{  on_pushButtonTracksEditRemoveAll_clicked();  }

void MainWindow::remove_tracks_nonrevised()
{  on_pushButtonTracksEditRemoveNonrevised_clicked();  }

int MainWindow::nuclei_count() const
{  assert(current_nuclei_);  return current_nuclei_->size();  }

int MainWindow::regions_count() const
{  assert(current_regions_);  return current_regions_->size();  }

long int MainWindow::frames_count() const
{  return n_frames_;  }

long int MainWindow::tracks_count() const
{  return tracks_.size();  }

long int MainWindow::current_frame() const
{  return current_frame_i_;  }

void MainWindow::go_to_frame(long int frame_i)
{
  assert(frame_i < INT_MAX);
  ui_->horizontalSlider->setValue(static_cast<int>(frame_i));
}

void MainWindow::echo(QString value)
{
  if (api_parameters.contains(value)) {
    message_('I', "[Script] " + get(value));
  } else {
    message_('I', "[Script] " + QString(value));
  }
}

void MainWindow::repaint()
{
  QApplication::sendPostedEvents();  // avoid image freeze
  QApplication::processEvents();     // avoid GUI freeze
  //QApplication::flush(); // <-- this can stall
}

void MainWindow::millisleep(int msecs) const
{
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
  QThread::msleep(static_cast<unsigned int>(msecs));
#else
  QMutex _(QMutex::NonRecursive);  _.lock();  _.tryLock(msecs);
#endif
}

void MainWindow::panic(QString reason)
{
  script_window_->abortExecution();
  batch_processing_ = false;

  message_('E', QString("[Script] ") + reason);

  QMessageBox msg;
   msg.setText("Script failure:");
   msg.setInformativeText(reason);
   msg.setStandardButtons(QMessageBox::Ok);
   msg.setDefaultButton(QMessageBox::Ok);
   msg.setIcon(QMessageBox::Critical);
   msg.exec();
}

void MainWindow::quit()
{  QApplication::quit();  }


void MainWindow::set(QString what, QString value)
{
  // TODO: Warn when a setting has ben trimmed to a valid spinBox range/rounded.

  QString const value_low = value.toLower();
  bool const value_on = QSet<QString>({"on", "enabled", "true", "yes", "1"})
                                     .contains(value_low);
  bool const value_off = not value_on;
  if (what == sParamNucDetStainingNuclear) {
    ui_->radioButtonNucDetChannelSelectionTypeNuclear->setChecked(value_on);
    assert(ui_->radioButtonNucDetChannelSelectionTypeCytoplasmic->isChecked() != value_on);
  } else if (what == sParamNucDetChannel) {
    bool set_success = false;
    for (int i=0; i < ui_->comboBoxNucDetChannelSelection->count(); ++i) {
      if (value == ui_->comboBoxNucDetChannelSelection->itemText(i)) {
        ui_->comboBoxNucDetChannelSelection->setCurrentIndex(i);
        set_success = true;
        break;
    } }
    if (not set_success) { panic(QString("No such channel: '")+value_low+"'."); }
  } else if (what == sParamNucDetNormalization) {
    ui_->checkBoxNucDetNormalization->setChecked(value_on);
  } else if (what == sParamNucDetDenoising) {
    ui_->checkBoxNucDetDenoising->setChecked(value_on);
  } else if (what == sParamNucDetDenoisingStrength) {
    ui_->dspinBoxNucDetDenoising->setValue(value.toDouble());
  } else if (what == sParamNucDetSmoothing) {
    ui_->checkBoxNucDetSmoothing->setChecked(value_on);
  } else if (what == sParamNucDetSmoothingRadius) {
    ui_->spinBoxNucDetSmoothing->setValue(value.toInt());
  } else if (what == sParamNucDetThresholding) {
    ui_->checkBoxNucDetThresholding->setChecked(value_on);
  } else if (what == sParamNucDetThresholdingBaseline or
             what == sParamNucDetThresholdingBaselineOBSOLETE) {
    ui_->spinBoxNucDetThresholdingBaseline->setValue(value.toInt());
  } else if (what == sParamNucDetThresholdingBlocksize or
             what == sParamNucDetThresholdingBlocksizeOBSOLETE) {
    ui_->spinBoxNucDetThresholdingBlocksize->setValue(value.toInt());
  } else if (what == sParamNucDetThresholdingAuto) {
    ui_->radioButtonNucDetParamsAuto  ->setChecked(value_on);
    ui_->radioButtonNucDetParamsManual->setChecked(value_off);
  } else if (what == sParamNucDetThresholdingBaselineAutoFrom) {
    ui_->spinBoxNucDetThresholdingBaselineAutoscanFrom->setValue(value.toInt());
  } else if (what == sParamNucDetThresholdingBaselineAutoUpto) {
    ui_->spinBoxNucDetThresholdingBaselineAutoscanUpto->setValue(value.toInt());
  } else if (what == sParamNucDetThresholdingBaselineAutoStep) {
    ui_->spinBoxNucDetThresholdingBaselineAutoscanStep->setValue(value.toInt());
  } else if (what == sParamNucDetThresholdingBlocksizeAutoFrom) {
    ui_->spinBoxNucDetThresholdingBlocksizeAutoscanFrom->setValue(value.toInt());
  } else if (what == sParamNucDetThresholdingBlocksizeAutoUpto) {
    ui_->spinBoxNucDetThresholdingBlocksizeAutoscanUpto->setValue(value.toInt());
  } else if (what == sParamNucDetThresholdingBlocksizeAutoNSteps) {
    ui_->spinBoxNucDetThresholdingBlocksizeAutoscanNsteps->setValue(value.toInt());
  } else if (what == sParamNucDetUniformization) {
    ui_->checkBoxNucDetEqualization->setChecked(value_on);
  } else if (what == sParamNucDetUniformizationThreshold or
             what == sParamNucDetUniformizationThresholdOBSOLETE) {
    ui_->spinBoxNucDetEqualizationThreshold->setValue(value.toInt());
  } else if (what == sParamNucDetOpening) {
    ui_->checkBoxNucDetOpening->setChecked(value_on);
  } else if (what == sParamNucDetOpeningRepeats) {
    ui_->spinBoxNucDetOpeningTimes->setValue(value.toInt());
  } else if (what == sParamNucDetOpeningErosionSize or
             what == sParamNucDetOpeningErosionSizeOBSOLETE) {
    ui_->spinBoxNucDetOpeningErosion->setValue(value.toInt());
  } else if (what == sParamNucDetOpeningDilationSize or
             what == sParamNucDetOpeningDilationSizeOBSOLETE) {
    ui_->spinBoxNucDetOpeningDilation->setValue(value.toInt());
  } else if (what == sParamNucDetClosing) {
    ui_->checkBoxNucDetClosing->setChecked(value_on);
  } else if (what == sParamNucDetClosingRepeats) {
    ui_->spinBoxNucDetClosingTimes->setValue(value.toInt());
  } else if (what == sParamNucDetClosingDilationSize or
             what == sParamNucDetClosingDilationSizeOBSOLETE) {
    ui_->spinBoxNucDetClosingDilation->setValue(value.toInt());
  } else if (what == sParamNucDetClosingErosionSize or
             what == sParamNucDetClosingErosionSizeOBSOLETE) {
    ui_->spinBoxNucDetClosingErosion->setValue(value.toInt());
  //
  } else if (what ==sParamNucDetContoursThresholdLow or
             what ==sParamNucDetContoursThresholdLowOBSOLETE) {
    ui_->spinBoxNucDetContourDetThresholdLow->setValue(value.toInt());
  } else if (what == sParamNucDetContoursThresholdsRatio or
             what == sParamNucDetContoursThresholdsRatioOBSOLETE) {
    ui_->dspinBoxNucDetContourDetThresholdsRatio->setValue(value.toDouble());
  } else if (what == sParamNucDetContoursTolerance or
             what == sParamNucDetContoursToleranceOBSOLETE) {
    ui_->dspinBoxNucDetContourDetTolerance->setValue(value.toDouble());
  } else if (what == sParamNucDetContoursReconstruct or
             what == sParamNucDetContoursReconstructOBSOLETE) {
    ui_->checkBoxNucDetContourDetReconstruct->setChecked(value_on);
  //
  } else if (what == sParamNucDetAssessmentMinSolidity or
             what == sParamNucDetAssessmentMinSolidityOBSOLETE) {
    ui_->dspinBoxNucDetSolidity->setValue(value.toDouble());
  } else if (what == sParamNucDetAssessmentAreaFracMin or
             what == sParamNucDetAssessmentAreaFracMinOBSOLETE) {
    ui_->dspinBoxNucDetMinAreaFrac->setValue(value.toDouble());
  } else if (what == sParamNucDetAssessmentAreaFracMax or
             what == sParamNucDetAssessmentAreaFracMaxOBSOLETE) {
    ui_->dspinBoxNucDetMaxAreaFrac->setValue(value.toDouble());
  //
  } else if (what == sParamNucDetSplitting or
             what == sParamNucDetSplittingOBSOLETE) {
    ui_->checkBoxNucDetSplitAdjacent->setChecked(value_on);
  } else if (what == sParamNucDetSplittingMinPocketArea or
             what == sParamNucDetSplittingMinPocketAreaOBSOLETE) {
    ui_->spinBoxNucDetMinPocketArea->setValue(value.toInt());
  //
  } else if (what == sParamNucEditNuccutChannelMain) {
    ui_->lineEditNucEditRefineNuccutChannelMain->setText(value);
  } else if (what == sParamNucEditNuccutChannelAux1) {
    bool set_success = false;
    for (int i=0; i < ui_->comboBoxNucEditRefineNuccutChannelAux1->count(); ++i) {
      if (value == ui_->comboBoxNucEditRefineNuccutChannelAux1->itemText(i)) {
        ui_->comboBoxNucEditRefineNuccutChannelAux1->setCurrentIndex(i);
        set_success = true;
    } }
    if (not set_success) { panic(QString("No such channel: '")+value_low+"'."); }
  } else if (what == sParamNucEditNuccutChannelAux2) {
    bool set_success = false;
    for (int i=0; i < ui_->comboBoxNucEditRefineNuccutChannelAux2->count(); ++i) {
      if (value == ui_->comboBoxNucEditRefineNuccutChannelAux2->itemText(i)) {
        ui_->comboBoxNucEditRefineNuccutChannelAux2->setCurrentIndex(i);
        set_success = true;
    } }
    if (not set_success) { panic(QString("No such channel: '")+value_low+"'."); }
  } else if (what == sParamNucEditNuccutChannelMainWeight) {
    ui_->spinBoxNucEditRefineNuccutChannelMain->setValue(value.toInt());
  } else if (what == sParamNucEditNuccutChannelAux1Weight) {
    ui_->spinBoxNucEditRefineNuccutChannelAux1->setValue(value.toInt());
  } else if (what == sParamNucEditNuccutChannelAux2Weight) {
    ui_->spinBoxNucEditRefineNuccutChannelAux2->setValue(value.toInt());
  } else if (what == sParamNucEditNuccutColorSimilarity) {
    ui_->dspinBoxNucEditRefineNuccutParamColorSimilarity->setValue(value.toDouble());
  } else if (what == sParamNucEditNuccutEdgeImpassability) {
    ui_->dspinBoxNucEditRefineNuccutParamEdgeImpassability->setValue(value.toDouble());
  } else if (what == sParamNucEditNuccutExcludeToggled) {
    ui_->checkBoxNucEditRefineNuccutExcludeToggled->setChecked(value.toInt());
  } else if (what == sParamNucEditShowingNucNumbers) {
    ui_->checkBoxNucEditShowAnnotations->setChecked(value_on);
  //
  } else if (what == sParamPerinucInnerOffset) {
    ui_->spinBoxPerinucSettingInnerOffset->setValue(value.toInt());
  } else if (what == sParamPerinucWidth) {
    ui_->spinBoxPerinucSettingWidth->setValue(value.toInt());
  } else if (what == sParamPerinucNeighAvoidance) {
    ui_->spinBoxPerinucSettingNeighAvoidance->setValue(value.toInt());
  } else if (what == sParamPerinucOverlapAvoidance) {
    ui_->spinBoxPerinucSettingOverlapAvoidance->setValue(value.toInt());
  } else if (what == sParamPerinucMinPartArea) {
    ui_->spinBoxPerinucSettingMinArea->setValue(value.toInt());
  } else if (what == sParamPerinucBgAvoid) {
    ui_->checkBoxPerinucSettingsBgAvoid->setChecked(value_on);
  } else if (what == sParamPerinucBgAvoidChannel) {
    bool set_success = false;
    for (int i=0; i < ui_->comboBoxPerinucSettingsBgAvoidChannel->count(); ++i) {
      if (value == ui_->comboBoxPerinucSettingsBgAvoidChannel->itemText(i)) {
        ui_->comboBoxPerinucSettingsBgAvoidChannel->setCurrentIndex(i);
        set_success = true;
    } }
    if (not set_success) { panic(QString("No such channel: '")+value_low+"'."); }
  } else if (what == sParamPerinucBgAvoidExpansion) {
    ui_->dspinBoxPerinucSettingBgAvoidExpansion->setValue(value.toDouble());
  } else if (what == sParamPerinucBgAvoidSmoothing) {
    ui_->spinBoxPerinucSettingBgAvoidSmoothing->setValue(value.toInt());
  //
  //
  } else if (what == sParamRgnEditBackgroundSubdivs) {
    ui_->spinBoxRgnEditBackgroundSubdivs->setValue(value.toInt());
  } else if (what == sParamRgnEditBackgroundCount) {
    ui_->spinBoxRgnEditBackgroundCount->setValue(value.toInt());
  } else if (what == sParamRgnEditBackgroundMergeAdj) {
    ui_->checkBoxRgnEditBackgroundMergeAdjacent->setChecked(value_on);
  } else if (what == sParamRgnEditVoronoiScaling) {
    ui_->checkBoxRgnEditVoronoiScaling->setChecked(value.toDouble() < 1.0);
    ui_->dspinBoxRgnEditVoronoiScaling->setValue(value.toDouble());
  } else if (what == sParamRgnEditVoronoiErosion) {
    ui_->checkBoxRgnEditVoronoiErosion->setChecked(value.toInt() > 0);
    ui_->spinBoxRgnEditVoronoiErosion->setValue(value.toInt());
  }  else if (what == sParamRgnEditVoronoiSavingNeighs) {
    ui_->checkBoxRgnEditVoronoiSaveNeighs->setChecked(value_on);
  } else if (what == sParamRgnEditShowingRegNumbers) {
    ui_->checkBoxRgnEditShowAnnotations->setChecked(value_on);
  //
  //
  } else if (what == sParamQuantChannelG) {
    if (images_->has(ImageChannel::G)) {
      ui_->checkBoxNucQuantChannelG->setChecked(value_on);
    } else {  panic("Green channel is unavailable.");  }
  } else if (what == sParamQuantChannelR) {
    if (images_->has(ImageChannel::R)) {
      ui_->checkBoxNucQuantChannelR->setChecked(value_on);
    } else {  panic("Red channel is unavailable.");  }
  } else if (what == sParamQuantChannelB) {
    if (images_->has(ImageChannel::B)) {
      ui_->checkBoxNucQuantChannelB->setChecked(value_on);
    } else {  panic("Blue channel is unavailable.");  }
    if (images_->has(ImageChannel::C)) {
      ui_->checkBoxNucQuantChannelC->setChecked(value_on);
    } else {  panic("Cyan channel is unavailable.");  }
  } else if (what == sParamQuantChannelM) {
    if (images_->has(ImageChannel::M)) {
      ui_->checkBoxNucQuantChannelM->setChecked(value_on);
    } else {  panic("Magenta channel is unavailable.");  }
  } else if (what == sParamQuantChannelY) {
    if (images_->has(ImageChannel::Y)) {
      ui_->checkBoxNucQuantChannelY->setChecked(value_on);
    } else {  panic("Yellow channel is unavailable.");  }
  //
  } else if (what == sParamQuantChannelDescG) {
    if (images_->has(ImageChannel::G)) {
      images_->setDescription(ImageChannel::G, value.toStdString());
      ui_->menuGreenChannel->setTitle(QString("Green channel: ") + value);
      ui_->lineEditQuantNucChannelG->setText(value);
    } else {  panic("Green channel is unavailable.");  }
  } else if (what == sParamQuantChannelDescR) {
    if (images_->has(ImageChannel::R)) {
      images_->setDescription(ImageChannel::R, value.toStdString());
      ui_->menuGreenChannel->setTitle(QString("Red channel: ") + value);
      ui_->lineEditQuantNucChannelR->setText(value);
    } else {  panic("Red channel is unavailable.");  }
  } else if (what == sParamQuantChannelDescB) {
    if (images_->has(ImageChannel::B)) {
      images_->setDescription(ImageChannel::B, value.toStdString());
      ui_->menuGreenChannel->setTitle(QString("Blue channel: ") + value);
      ui_->lineEditQuantNucChannelB->setText(value);
    } else {  panic("Blue channel is unavailable.");  }
  } else if (what == sParamQuantChannelDescC) {
    if (images_->has(ImageChannel::C)) {
      images_->setDescription(ImageChannel::C, value.toStdString());
      ui_->menuGreenChannel->setTitle(QString("Cyan channel: ") + value);
      ui_->lineEditQuantNucChannelC->setText(value);
    } else {  panic("Cyan channel is unavailable.");  }
  } else if (what == sParamQuantChannelDescM) {
    if (images_->has(ImageChannel::M)) {
      images_->setDescription(ImageChannel::M, value.toStdString());
      ui_->menuGreenChannel->setTitle(QString("Magenta channel: ") + value);
      ui_->lineEditQuantNucChannelM->setText(value);
    } else {  panic("Magenta channel is unavailable.");  }
  } else if (what == sParamQuantChannelDescY) {
    if (images_->has(ImageChannel::Y)) {
      images_->setDescription(ImageChannel::Y, value.toStdString());
      ui_->menuGreenChannel->setTitle(QString("Yellow channel: ") + value);
      ui_->lineEditQuantNucChannelY->setText(value);
    } else {  panic("Yellow channel is unavailable.");  }
  //
  } else if (what == sParamQuantImgExportDat) {
     ui_->checkBoxQuantImgFilesDat->setChecked(value_on);
  } else if (what == sParamQuantImgOnlyNonmasked) {
     ui_->checkBoxQuantImgOnlyNonMasked->setChecked(value_on);
  } else if (what == sParamQuantNucExportDat) {
    ui_->checkBoxNucQuantSaveCsv->setChecked(value_on);
  } else if (what == sParamQuantNucExportPng) {
    ui_->checkBoxNucQuantSavePng->setChecked(value_on);
  } else if (what == sParamQuantPerExportDat) {
    ui_->checkBoxNucQuantSavePng->setChecked(value_on);
  } else if (what == sParamQuantPerExportHalos) {
    ui_->checkBoxQuantNucSaveHalos->setChecked(value_on);
  } else if (what == sParamQuantRgnExportDat) {
    ui_->checkBoxQuantRgnSaveDat->setChecked(value_on);
  } else if (what == sParamQuantRgnExportPng) {
    ui_->checkBoxQuantRgnSavePng->setChecked(value_on);
  //
  //
  } else if (what == sParamTrackingNucPosPrediction) {
    ui_->groupBoxTrackingNucPositionPrediction->setChecked(value_on);
  } else if (what == sParamTrackingNucPosPredictionMemory) {
    ui_->spinBoxTrackingNucPositionPredictionHistory->setValue(value.toInt());
  } else if (what == sParamTrackingNucPosPredictionConservation) {
    ui_->dspinBoxTrackingNucPositionPredictionConservation->setValue(value.toDouble());
  } else if (what == sParamTrackingNucPosPredictionContribution) {
    ui_->dspinBoxTrackingNucPositionPredictionContribution->setValue(value.toDouble());
  //
  } else if (what == sParamTrackingNucSimWeightProximity) {
    ui_->dspinBoxTrackingNucSimilarityWeightProximity->setValue(value.toDouble());
  } else if (what == sParamTrackingNucSimWeightArea) {
    ui_->dspinBoxTrackingNucSimilarityWeightArea->setValue(value.toDouble());
  } else if (what == sParamTrackingNucSimWeightEccentricity) {
    ui_->dspinBoxTrackingNucSimilarityWeightEccentricity->setValue(value.toDouble());
  } else if (what == sParamTrackingNucSimWeightOrientation) {
    ui_->dspinBoxTrackingNucSimilarityWeightOrientation->setValue(value.toDouble());
  } else if (what == sParamTrackingNucSimWeightIntensitySum) {
    ui_->dspinBoxTrackingNucSimilarityWeightIntensitySum->setValue(value.toDouble());
  } else if (what == sParamTrackingNucSimWeightIntensityInertia) {
    ui_->dspinBoxTrackingNucSimilarityWeightIntensityInertia->setValue(value.toDouble());
  } else if (what == sParamTrackingNucSimWeightIntensityDistrib) {
    ui_->dspinBoxTrackingNucSimilarityWeightIntensityDistrib->setValue(value.toDouble());
  } else if (what == sParamTrackingNucFeatureDistScaling) {
    ui_->dspinBoxTrackingNucDistanceScaling->setValue(value.toDouble());
  } else if (what == sParamTrackingNucDisplacementCutoff) {
    ui_->dspinBoxTrackingNucDisplacementCutoffMedian->setValue(value.toDouble());
  } else if (what == sParamTrackingNucAreaDrop) {
    ui_->groupBoxTrackingTrackBreakUponNucAreaDropRelative->setChecked(value_on);
  } else if (what == sParamTrackingNucAreaDropThreshold) {
    ui_->dspinBoxTrackingTrackBreakUponNucAreaDropRelative->setValue(value.toDouble());
  //
  } else if (what == sParamTracksEditSaveOnlyRevised) {
    ui_->checkBoxTracksEditSaveOnlyRevisedTracks->setChecked(value_on);
  //
  } else {
    panic(QString("No such parameter to set: '") + value + "'.");
  }
}

QString MainWindow::get(QString what)
{
  QString const on = "on", off = "off";

  if (what == sParamNucDetChannel) {
    return ui_->comboBoxNucDetChannelSelection->currentText();
  } else if (what == sParamNucDetNormalization) {
    return ui_->checkBoxNucDetNormalization->isChecked() ? on : off;
  } else if (what == sParamNucDetDenoising) {
    return ui_->checkBoxNucDetDenoising->isChecked() ? on : off;
  } else if (what == sParamNucDetDenoisingStrength) {
    return QString::number(ui_->dspinBoxNucDetDenoising->value());
  } else if (what == sParamNucDetSmoothing) {
    return ui_->checkBoxNucDetSmoothing->isChecked() ? on : off;
  } else if (what == sParamNucDetSmoothingRadius) {
    return QString::number(ui_->spinBoxNucDetSmoothing->value());
  } else if (what == sParamNucDetThresholding) {
    return ui_->checkBoxNucDetThresholding->isChecked() ? on : off;
  } else if (what == sParamNucDetThresholdingBaseline) {
    return QString::number(ui_->spinBoxNucDetThresholdingBaseline->value());
  } else if (what == sParamNucDetThresholdingBlocksize) {
    return QString::number(ui_->spinBoxNucDetThresholdingBlocksize->value());
  } else if (what == sParamNucDetThresholdingAuto) {
    assert(   ui_->radioButtonNucDetParamsManual->isChecked()
           != ui_->radioButtonNucDetParamsAuto->isChecked()) ;
    return ui_->radioButtonNucDetParamsAuto->isChecked() ? on : off;
  } else if (what == sParamNucDetThresholdingBaselineAutoFrom) {
    return QString::number(ui_->spinBoxNucDetThresholdingBaselineAutoscanFrom->value());
  } else if (what == sParamNucDetThresholdingBaselineAutoUpto) {
    return QString::number(ui_->spinBoxNucDetThresholdingBaselineAutoscanUpto->value());
  } else if (what == sParamNucDetThresholdingBaselineAutoStep) {
    return QString::number(ui_->spinBoxNucDetThresholdingBaselineAutoscanStep->value());
  } else if (what == sParamNucDetThresholdingBlocksizeAutoFrom) {
    return QString::number(ui_->spinBoxNucDetThresholdingBlocksizeAutoscanFrom->value());
  } else if (what == sParamNucDetThresholdingBlocksizeAutoUpto) {
    return QString::number(ui_->spinBoxNucDetThresholdingBlocksizeAutoscanUpto->value());
  } else if (what == sParamNucDetThresholdingBlocksizeAutoNSteps) {
    return QString::number(ui_->spinBoxNucDetThresholdingBlocksizeAutoscanNsteps->value());
  } else if (what ==sParamNucDetUniformization) {
    return ui_->checkBoxNucDetEqualization->isChecked() ? on : off;
  } else if (what ==sParamNucDetUniformizationThreshold) {
    return QString::number(ui_->spinBoxNucDetEqualizationThreshold->value());
  } else if (what == sParamNucDetOpening) {
    return ui_->checkBoxNucDetOpening->isChecked() ? on : off;
  } else if (what == sParamNucDetOpeningRepeats) {
    return QString::number(ui_->spinBoxNucDetOpeningTimes->value());
  } else if (what == sParamNucDetOpeningErosionSize) {
    return QString::number(ui_->spinBoxNucDetOpeningErosion->value());
  } else if (what == sParamNucDetOpeningDilationSize) {
    return QString::number(ui_->spinBoxNucDetOpeningDilation->value());
  } else if (what == sParamNucDetClosing) {
    return ui_->checkBoxNucDetClosing->isChecked() ? on : off;
  } else if (what == sParamNucDetClosingRepeats) {
    return QString::number(ui_->spinBoxNucDetClosingTimes->value());
  } else if (what == sParamNucDetClosingDilationSize) {
    return QString::number(ui_->spinBoxNucDetClosingDilation->value());
  } else if (what == sParamNucDetClosingErosionSize) {
    return QString::number(ui_->spinBoxNucDetClosingErosion->value());
  //
  } else if (what ==sParamNucDetContoursThresholdLow) {
    return  QString::number(ui_->spinBoxNucDetContourDetThresholdLow->value());
  } else if (what == sParamNucDetContoursThresholdsRatio) {
    return  QString::number(ui_->dspinBoxNucDetContourDetThresholdsRatio->value());
  } else if (what == sParamNucDetContoursTolerance) {
    return  QString::number(ui_->dspinBoxNucDetContourDetTolerance->value());
  } else if (what == sParamNucDetContoursReconstruct) {
    return ui_->checkBoxNucDetContourDetReconstruct->isChecked() ? on : off;
  //
  } else if (what == sParamNucDetAssessmentMinSolidity) {
    return QString::number(ui_->dspinBoxNucDetSolidity->value());
  } else if (what == sParamNucDetAssessmentAreaFracMin) {
    return QString::number(ui_->dspinBoxNucDetMinAreaFrac->value());
  } else if (what == sParamNucDetAssessmentAreaFracMax) {
    return QString::number(ui_->dspinBoxNucDetMaxAreaFrac->value());
  //
  } else if (what == sParamNucDetSplitting) {
    return ui_->checkBoxNucDetSplitAdjacent->isChecked() ? on : off;
  } else if (what == sParamNucDetSplittingMinPocketArea) {
    return QString::number(ui_->spinBoxNucDetMinPocketArea->value());
  //
  //
  } else if (what == sParamNucEditNuccutChannelMain) {
    return ui_->lineEditNucEditRefineNuccutChannelMain->text();
  } else if (what == sParamNucEditNuccutChannelAux1) {
    return ui_->comboBoxNucEditRefineNuccutChannelAux1->currentText();
  } else if (what == sParamNucEditNuccutChannelAux2) {
    return ui_->comboBoxNucEditRefineNuccutChannelAux2->currentText();
  } else if (what == sParamNucEditNuccutChannelMainWeight) {
    return QString::number(ui_->spinBoxNucEditRefineNuccutChannelMain->value());
  } else if (what == sParamNucEditNuccutChannelAux1Weight) {
    return QString::number(ui_->spinBoxNucEditRefineNuccutChannelAux1->value());
  } else if (what == sParamNucEditNuccutChannelAux2Weight) {
    return QString::number(ui_->spinBoxNucEditRefineNuccutChannelAux2->value());
  } else if (what == sParamNucEditNuccutColorSimilarity) {
    return QString::number(ui_->dspinBoxNucEditRefineNuccutParamColorSimilarity
                              ->value());
  } else if (what == sParamNucEditNuccutEdgeImpassability) {
    return QString::number(ui_->dspinBoxNucEditRefineNuccutParamEdgeImpassability
                              ->value());
  } else if (what == sParamNucEditNuccutExcludeToggled) {
    return ui_->checkBoxNucEditRefineNuccutExcludeToggled->isChecked()? on: off;
  } else if (what == sParamNucEditShowingNucNumbers) {
    return ui_->checkBoxNucEditShowAnnotations->isChecked() ? on : off;
  //
  //
  } else if (what == sParamPerinucInnerOffset) {
    return QString::number(ui_->spinBoxPerinucSettingInnerOffset->value());
  } else if (what == sParamPerinucWidth) {
    return QString::number(ui_->spinBoxPerinucSettingWidth->value());
  } else if (what == sParamPerinucNeighAvoidance) {
    return QString::number(ui_->spinBoxPerinucSettingNeighAvoidance->value());
  } else if (what == sParamPerinucOverlapAvoidance) {
    return QString::number(ui_->spinBoxPerinucSettingOverlapAvoidance->value());
  } else if (what == sParamPerinucMinPartArea) {
    return QString::number(ui_->spinBoxPerinucSettingMinArea->value());
  } else if (what == sParamPerinucBgAvoid) {
    return ui_->checkBoxPerinucSettingsBgAvoid->isChecked() ? on : off;
  } else if (what == sParamPerinucBgAvoidChannel) {
    return ui_->comboBoxPerinucSettingsBgAvoidChannel->currentText();
  } else if (what == sParamPerinucBgAvoidExpansion) {
    return QString::number(ui_->dspinBoxPerinucSettingBgAvoidExpansion->value());
  } else if (what == sParamPerinucBgAvoidSmoothing) {
    return QString::number(ui_->spinBoxPerinucSettingBgAvoidSmoothing->value());
  //
  //
  } else if (what == sParamRgnEditBackgroundSubdivs) {
    return QString::number(ui_->spinBoxRgnEditBackgroundSubdivs->value());
  } else if (what == sParamRgnEditBackgroundCount) {
    return QString::number(ui_->spinBoxRgnEditBackgroundCount->value());
  } else if (what == sParamRgnEditBackgroundMergeAdj) {
    return ui_->checkBoxRgnEditBackgroundMergeAdjacent->isChecked() ? on : off;
  } else if (what == sParamRgnEditVoronoiScaling) {
    assert(ui_->dspinBoxRgnEditVoronoiScaling->value() > 0 ?
           ui_->checkBoxRgnEditVoronoiScaling->isChecked() : true);
    return QString::number(ui_->dspinBoxRgnEditVoronoiScaling->value());
  } else if (what == sParamRgnEditVoronoiErosion) {
    assert(ui_->spinBoxRgnEditVoronoiErosion->value() > 0 ?
           ui_->checkBoxRgnEditVoronoiErosion->isChecked() : true);
    return QString::number(ui_->spinBoxRgnEditVoronoiErosion->value());
  }  else if (what == sParamRgnEditVoronoiSavingNeighs) {
    return ui_->checkBoxRgnEditVoronoiSaveNeighs->isChecked() ? on : off;
  } else if (what == sParamRgnEditShowingRegNumbers) {
    return ui_->checkBoxRgnEditShowAnnotations->isChecked() ? on : off;
  //
  //
  } else if (what == sParamQuantChannelG) {
    //if (ui_->checkBoxNucQuantChannelG->isVisible()) {
      return ui_->checkBoxNucQuantChannelG->isChecked() ? on : off;
    //} else {  panic("Green channel is unavailable.");  }
  } else if (what == sParamQuantChannelR) {
    //if (ui_->checkBoxNucQuantChannelR->isVisible()) {
      return ui_->checkBoxNucQuantChannelR->isChecked() ? on : off;
    //} else {  panic("Red channel is unavailable.");  }
  } else if (what == sParamQuantChannelB) {
    //if (ui_->checkBoxNucQuantChannelB->isVisible()) {
      return ui_->checkBoxNucQuantChannelB->isChecked() ? on : off;
    //} else {  panic("Blue channel is unavailable.");  }
  } else if (what == sParamQuantChannelC) {
    //if (ui_->checkBoxNucQuantChannelC->isVisible()) {
      return ui_->checkBoxNucQuantChannelC->isChecked() ? on : off;
    //} else {  panic("Cyan channel is unavailable.");  }
  } else if (what == sParamQuantChannelM) {
    //if (ui_->checkBoxNucQuantChannelM->isVisible()) {
      return ui_->checkBoxNucQuantChannelM->isChecked() ? on : off;
    //} else {  panic("Magenta channel is unavailable.");  }
  } else if (what == sParamQuantChannelY) {
    //if (ui_->checkBoxNucQuantChannelY->isVisible()) {
      return ui_->checkBoxNucQuantChannelY->isChecked() ? on : off;
    //} else {  panic("Yellow channel is unavailable.");  }
  //
  } else if (what == sParamQuantChannelDescG) {
    if (ui_->checkBoxNucQuantChannelG->isVisible()) {
      return ui_->lineEditQuantNucChannelG->text();
    } else {  panic("Green channel is unavailable.");  return "?";  }
  } else if (what == sParamQuantChannelDescR) {
    if (ui_->checkBoxNucQuantChannelR->isVisible()) {
      return ui_->lineEditQuantNucChannelR->text();
    } else {  panic("Red channel is unavailable.");  return "?";  }
  } else if (what == sParamQuantChannelDescB) {
    if (ui_->checkBoxNucQuantChannelB->isVisible()) {
      return ui_->lineEditQuantNucChannelB->text();
    } else {  panic("Blue channel is unavailable.");  return "?";  }
  } else if (what == sParamQuantChannelDescC) {
    if (ui_->checkBoxNucQuantChannelC->isVisible()) {
      return ui_->lineEditQuantNucChannelC->text();
    } else {  panic("Cyan channel is unavailable.");  return "?";  }
  } else if (what == sParamQuantChannelDescM) {
    if (ui_->checkBoxNucQuantChannelM->isVisible()) {
      return ui_->lineEditQuantNucChannelM->text();
    } else {  panic("Magenta channel is unavailable.");  return "?";  }
  } else if (what == sParamQuantChannelDescY) {
    if (ui_->checkBoxNucQuantChannelY->isVisible()) {
      return ui_->lineEditQuantNucChannelY->text();
    } else {  panic("Yellow channel is unavailable.");  return "?";  }
  //
  } else if (what == sParamQuantImgExportDat) {
    return ui_->checkBoxQuantImgFilesDat->isChecked() ? on : off;
  } else if (what == sParamQuantImgOnlyNonmasked) {
    return ui_->checkBoxQuantImgOnlyNonMasked->isChecked() ? on : off;
  } else if (what == sParamQuantNucExportDat) {
    return ui_->checkBoxNucQuantSaveCsv->isChecked() ? on : off;
  } else if (what == sParamQuantNucExportPng) {
    return ui_->checkBoxNucQuantSavePng->isChecked() ? on : off;
  } else if (what == sParamQuantPerExportDat) {
    return ui_->checkBoxQuantPerinucSaveCsv->isChecked() ? on : off;
  } else if (what == sParamQuantRgnExportDat) {
    return ui_->checkBoxQuantRgnSaveDat->isChecked() ? on : off;
  } else if (what == sParamQuantRgnExportPng) {
    return ui_->checkBoxQuantRgnSavePng->isChecked() ? on : off;
  //
  //
  } else if (what == sParamTrackingNucPosPrediction) {
    return ui_->groupBoxTrackingNucPositionPrediction->isChecked() ? on : off;
  } else if (what == sParamTrackingNucPosPredictionMemory) {
    return QString::number(ui_->spinBoxTrackingNucPositionPredictionHistory->value());
  } else if (what == sParamTrackingNucPosPredictionConservation) {
    return QString::number(ui_->dspinBoxTrackingNucPositionPredictionConservation->value());
  } else if (what == sParamTrackingNucPosPredictionContribution) {
    return QString::number(ui_->dspinBoxTrackingNucPositionPredictionContribution->value());
  //
  } else if (what == sParamTrackingNucSimWeightProximity) {
    return QString::number(ui_->dspinBoxTrackingNucSimilarityWeightProximity->value());
  } else if (what == sParamTrackingNucSimWeightArea) {
    return QString::number(ui_->dspinBoxTrackingNucSimilarityWeightArea->value());
  } else if (what == sParamTrackingNucSimWeightEccentricity) {
    return QString::number(ui_->dspinBoxTrackingNucSimilarityWeightEccentricity->value());
  } else if (what == sParamTrackingNucSimWeightOrientation) {
    return QString::number(ui_->dspinBoxTrackingNucSimilarityWeightOrientation->value());
  } else if (what == sParamTrackingNucSimWeightIntensitySum) {
    return QString::number(ui_->dspinBoxTrackingNucSimilarityWeightIntensitySum->value());
  } else if (what == sParamTrackingNucSimWeightIntensityInertia) {
    return QString::number(ui_->dspinBoxTrackingNucSimilarityWeightIntensityInertia->value());
  } else if (what == sParamTrackingNucSimWeightIntensityDistrib) {
    return QString::number(ui_->dspinBoxTrackingNucSimilarityWeightIntensityDistrib->value());
  } else if (what == sParamTrackingNucFeatureDistScaling) {
    return QString::number(ui_->dspinBoxTrackingNucDistanceScaling->value());
  } else if (what == sParamTrackingNucDisplacementCutoff) {
    return QString::number(ui_->dspinBoxTrackingNucDisplacementCutoffMedian->value());
  } else if (what == sParamTrackingNucAreaDrop) {
    return ui_->groupBoxTrackingTrackBreakUponNucAreaDropRelative->isChecked() ? on : off;
  } else if (what == sParamTrackingNucAreaDropThreshold) {
    return QString::number(ui_->dspinBoxTrackingTrackBreakUponNucAreaDropRelative->value());
  //
  } else if (what == sParamTracksEditSaveOnlyRevised) {
    return ui_->checkBoxTracksEditSaveOnlyRevisedTracks->isChecked() ? on : off;
  //
  } else {
    panic(QString("No such parameter to get: '") + what + "'.");
  }
  return "?";
}

// -------------


void MainWindow::postLoadScript(QString const& script)
{
  post_images_load_hook_script_ = script;
  ui_->actionAfterOpenCachePixmaps->setChecked(false);
}

void MainWindow::setQuitUponScriptError(bool whether)
{  should_quit_upon_script_error_ = whether;  }

bool MainWindow::shouldQuitUponScriptError() const
{  return should_quit_upon_script_error_;  }

void MainWindow::___batch_processing___(bool on)
{
  if (batch_processing_ and (not on)) {
    batch_processing_ = on;
    message_('I', "[Batch processing has finished.]", 3000);
  }
  batch_processing_ = on;
}


void MainWindow::toggleToolboxenEnablednessForScriptExecution(bool on)
{
  for (auto& tb: {ui_->toolBoxTool0, ui_->toolBoxTool1,
                  ui_->toolBoxTool2, ui_->toolBoxTool3,
                  ui_->toolBoxTool4, ui_->toolBoxTool5,
                  ui_->toolBoxTool6, ui_->toolBoxTool7}) {
    tb->setEnabled(on);
    if (toolboxen_external_widgets_.contains(tb)) {
      auto& tb_ws = *toolboxen_external_widgets_.find(tb);
      for (auto& tb_w: tb_ws) {
        tb_w->setEnabled(on);
    } }
  } // for tb
}
