// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#include "MainWindow.hpp"
#include "ui_MainWindow.h"


bool MainWindow::nucleiDetectionParameterScanSettingsAreCorrect() /*const*/
{
  QString const msg_begin = "Cannot perform parameter scan: ";

  if (  ui_->spinBoxNucDetThresholdingBlocksizeAutoscanFrom->value()
      > ui_->spinBoxNucDetThresholdingBlocksizeAutoscanUpto->value()) {
    message_('E', msg_begin +"Incorrect blocksize range: from > up to.");
    return false;
  }

  if ((   ui_->spinBoxNucDetThresholdingBlocksizeAutoscanFrom->value()
       == ui_->spinBoxNucDetThresholdingBlocksizeAutoscanUpto->value())
      and
      (1< ui_->spinBoxNucDetThresholdingBlocksizeAutoscanNsteps->value())) {
     message_('E', msg_begin + "Incorrect blocksize stepping: from == up to, "
                               "while the number steps > 1.");
    return false;
  }

  if (   ui_->spinBoxNucDetThresholdingBaselineAutoscanFrom->value()
       > ui_->spinBoxNucDetThresholdingBaselineAutoscanUpto->value()) {
    message_('E', msg_begin + "Incorrect baseline range: from > up to.");
    return false;
  }

  if ( (   ui_->spinBoxNucDetThresholdingBaselineAutoscanFrom->value()
        == ui_->spinBoxNucDetThresholdingBaselineAutoscanUpto->value())
      and
       (0< ui_->spinBoxNucDetThresholdingBaselineAutoscanStep->value())) {
    message_('E', msg_begin + "Incorrect baseline stepping: from == up to, "
                              "while step > 0.");
    return false;
  }

  if ( (0 == ui_->spinBoxNucDetThresholdingBaselineAutoscanStep->value())
      and
       (   ui_->spinBoxNucDetThresholdingBaselineAutoscanFrom->value()
        != ui_->spinBoxNucDetThresholdingBaselineAutoscanUpto->value())) {
    message_('E', msg_begin + "Incorrect baseline stepping: step is 0, "
                              "while there is more than one value in the range.");
    return false;
  }

  return true;
}


// TODO: is this type alias required?
st::ImagePreprocessingParameters
MainWindow::getImagePreprocessingParameters() const
{
  return {
      ui_->checkBoxNucDetNormalization->isChecked(),
      ui_->checkBoxNucDetDenoising->isChecked(),
          ui_->dspinBoxNucDetDenoising->value(),
      ui_->checkBoxNucDetSmoothing->isChecked(),
          ui_->spinBoxNucDetSmoothing->value(),
      ui_->checkBoxNucDetThresholding->isChecked(),
          ui_->spinBoxNucDetThresholdingBlocksize->value(),
          ui_->spinBoxNucDetThresholdingBaseline->value(),
      ui_->checkBoxNucDetOpening->isChecked(),
          ui_->spinBoxNucDetOpeningTimes->value(),
          ui_->spinBoxNucDetOpeningErosion->value(),
          ui_->spinBoxNucDetOpeningDilation->value(),
      ui_->checkBoxNucDetClosing->isChecked(),
          ui_->spinBoxNucDetClosingTimes->value(),
          ui_->spinBoxNucDetClosingDilation->value(),
          ui_->spinBoxNucDetClosingErosion->value()};
}

st::NucleiPreselectionParameters
MainWindow::getNucleiPreselectionParameters() const
{
  return {
      ui_->dspinBoxNucDetSolidity->value(),
      ui_->dspinBoxNucDetMinAreaFrac->value(),
      ui_->dspinBoxNucDetMaxAreaFrac->value(),
      ui_->spinBoxNucDetMinPocketArea->value()};
}

st::stained::NucleiDetectionParameters
MainWindow::getStainedNucleiDetectionParameters() const
{
  return {ui_->checkBoxNucDetEqualization->isChecked(),
          ui_->spinBoxNucDetEqualizationThreshold->value()};
}

st::nonstained::NucleiDetectionParameters
MainWindow::getNonstainedNucleiDetectionParameters() const
{
  return {
      ui_->spinBoxNucDetContourDetThresholdLow->value(),
      static_cast<float>(ui_->dspinBoxNucDetContourDetThresholdsRatio->value()),
      static_cast<float>(ui_->dspinBoxNucDetContourDetTolerance->value()),
      ui_->checkBoxNucDetContourDetReconstruct->isChecked() };
}


// == nuclear channel selection ================================================

void MainWindow::on_radioButtonNucDetChannelSelectionTypeNuclear_toggled(bool checked)
{
  assert(ui_->radioButtonNucDetChannelSelectionTypeCytoplasmic->isChecked() != checked);

  ui_->groupBoxNucDetBlobDet->setVisible(checked);
  ui_->groupBoxNucDetEdgeDet->setVisible(not checked);

  // TODO:REPETITION:4bc8b312
  ui_->checkBoxNucDetClosing->setVisible(
      ui_->radioButtonNucDetChannelSelectionTypeCytoplasmic->isChecked());
  ui_->spinBoxNucDetClosingTimes->setVisible(
      ui_->radioButtonNucDetChannelSelectionTypeCytoplasmic->isChecked());
  ui_->labelNucDetClosingDilation->setVisible(
      ui_->radioButtonNucDetChannelSelectionTypeCytoplasmic->isChecked());
  ui_->spinBoxNucDetClosingDilation->setVisible(
      ui_->radioButtonNucDetChannelSelectionTypeCytoplasmic->isChecked());
  ui_->labelNucDetClosingErosion->setVisible(
      ui_->radioButtonNucDetChannelSelectionTypeCytoplasmic->isChecked());
  ui_->spinBoxNucDetClosingErosion->setVisible(
      ui_->radioButtonNucDetChannelSelectionTypeCytoplasmic->isChecked());

  ui_->dspinBoxNucDetDenoising->setDecimals(checked ? 1 : 0);
  ui_->dspinBoxNucDetDenoising->setSingleStep(checked ? 0.5 : 1.0);

  ui_->groupBoxNucEditRefineNuccut->setEnabled(checked);

  /*
  ui_->checkBoxNucDetThresholding->setVisible(checked);

  ui_->groupBoxNucDetThresholdingBaselineAutoscanParams->setVisible(checked);
  ui_->groupBoxNucDetThresholdingBlocksizeAutoscanParams->setVisible(checked);

  ui_->radioButtonNucDetParamsAuto->setVisible(checked);
  ui_->radioButtonNucDetParamsManual->setVisible(checked);
  //
  ui_->labelNucDetThresholdingBlocksize->setVisible(checked);
  ui_->spinBoxNucDetThresholdingBlocksize->setVisible(checked);
  ui_->spinBoxNucDetThresholdingBlocksizeAutoscanFrom->setVisible(checked);
  ui_->spinBoxNucDetThresholdingBlocksizeAutoscanNsteps->setVisible(checked);
  ui_->spinBoxNucDetThresholdingBlocksizeAutoscanUpto->setVisible(checked);
  ui_->labelNucDetThresholdingBlocksize->setVisible(checked);
  ui_->labelNucDetThresholdingBlocksizeAutoscanFrom->setVisible(checked);
  ui_->labelNucDetThresholdingBlocksizeAutoscanNsteps->setVisible(checked);
  ui_->labelNucDetThresholdingBlocksizeAutoscanUpto->setVisible(checked);

  ui_->labelNucDetThresholdingBaseline->setVisible(checked);
  ui_->spinBoxNucDetThresholdingBaseline->setVisible(checked);
  ui_->spinBoxNucDetThresholdingBaselineAutoscanFrom->setVisible(checked);
  ui_->spinBoxNucDetThresholdingBaselineAutoscanStep->setVisible(checked);
  ui_->spinBoxNucDetThresholdingBaselineAutoscanUpto->setVisible(checked);
  ui_->labelNucDetThresholdingBaseline->setVisible(checked);
  ui_->labelNucDetThresholdingBaselineAutoscanFrom->setVisible(checked);
  ui_->labelNucDetThresholdingBaselineAutoscanEvery->setVisible(checked);
  ui_->labelNucDetThresholdingBaselineAutoscanUpto->setVisible(checked);

  ui_->labelNucDetThresholdingAutoscanObjective->setVisible(checked);
  ui_->comboBoxNucDetThresholdingAutoscanObjective->setVisible(checked);
  ui_->labelNucDetThresholdingThreadTimeout->setVisible(checked);
  ui_->dspinBoxNucDetThresholdingThreadTimeout->setVisible(checked);

  ui_->checkBoxNucDetOpening->setVisible(checked);
  ui_->spinBoxNucDetOpeningTimes->setVisible(checked);
  ui_->labelNucDetOpeningDilation->setVisible(checked);
  ui_->spinBoxNucDetOpeningDilation->setVisible(checked);
  ui_->labelNucDetOpeningErosion->setVisible(checked);
  ui_->spinBoxNucDetOpeningErosion->setVisible(checked);

  ui_->checkBoxNucDetClosing->setVisible(not checked);
  ui_->spinBoxNucDetClosingTimes->setVisible(not checked);
  ui_->labelNucDetClosingDilation->setVisible(not checked);
  ui_->spinBoxNucDetClosingDilation->setVisible(not checked);
  ui_->labelNucDetClosingErosion->setVisible(not checked);
  ui_->spinBoxNucDetClosingErosion->setVisible(not checked);

  ui_->spinBoxNucDetContourDetThresholdLow->setVisible(not checked);
  ui_->labelNucDetContourDetThresholdLow->setVisible(not checked);
  ui_->dspinBoxNucDetContourDetThresholdsRatio->setVisible(not checked);
  ui_->labelNucDetContourDetThresholdsRatio->setVisible(not checked);
  ui_->dspinBoxNucDetContourDetTolerance->setVisible(not checked);
  ui_->labelNucDetContourDetTolerance->setVisible(not checked);
  ui_->checkBoxNucDetContourDetReconstruct->setVisible(not checked);
  */
}


// == group box: image preprocessing ===========================================

// -- normalization --

void MainWindow::on_checkBoxNucDetNormalization_toggled(bool /*checked*/)
{ maybe_NucDetDetect_autoclick(); }


// -- denoising --

void MainWindow::on_checkBoxNucDetDenoising_toggled(bool checked)
{
  ui_->dspinBoxNucDetDenoising->setEnabled(checked);
  maybe_NucDetDetect_autoclick();
}

void MainWindow::on_dspinBoxNucDetDenoising_valueChanged(QString const& arg)
{ if (arg.size() > 0) { on_dspinBoxNucDetDenoising_editingFinished (); }  }

void MainWindow::on_dspinBoxNucDetDenoising_editingFinished()
{ maybe_NucDetDetect_autoclick(); }


// -- smoothing --

void MainWindow::on_checkBoxNucDetSmoothing_toggled(bool checked)
{
  ui_->spinBoxNucDetSmoothing->setEnabled(checked);
  maybe_NucDetDetect_autoclick();
}

void MainWindow::on_spinBoxNucDetSmoothing_editingFinished()
{ maybe_NucDetDetect_autoclick(); }

void MainWindow::on_spinBoxNucDetSmoothing_valueChanged(QString const&arg)
{ if (arg.size() > 0) { on_spinBoxNucDetSmoothing_editingFinished(); } }


// -- thresholding --

void MainWindow::on_checkBoxNucDetThresholding_toggled(bool checked)
{
  ui_->  labelNucDetThresholdingBaseline->setEnabled(checked);
  ui_->spinBoxNucDetThresholdingBaseline->setEnabled(checked);
  ui_->  labelNucDetThresholdingBlocksize->setEnabled(checked);
  ui_->spinBoxNucDetThresholdingBlocksize->setEnabled(checked);
  ui_->radioButtonNucDetParamsAuto->setEnabled(checked);
  ui_->radioButtonNucDetParamsManual->setEnabled(checked);
  maybe_NucDetDetect_autoclick();
}

void MainWindow::on_spinBoxNucDetThresholdingBlocksize_editingFinished()
{
  // assure odd:
  auto const v = ui_->spinBoxNucDetThresholdingBlocksize->value();
  if (not (v % 2)) {
    ui_->spinBoxNucDetThresholdingBlocksize->setValue(v + 1);
  }
  maybe_NucDetDetect_autoclick();
}

void MainWindow::on_spinBoxNucDetThresholdingBlocksize_valueChanged(const QString& arg)
{ if (arg.size() > 0) { on_spinBoxNucDetThresholdingBlocksize_editingFinished(); } }

void MainWindow::on_spinBoxNucDetThresholdingBaseline_editingFinished()
{
  ui_->spinBoxNucDetThresholdingBaseline->setPrefix(
      (ui_->spinBoxNucDetThresholdingBaseline->value() > 0) ? "+" : " ");
  maybe_NucDetDetect_autoclick();
}

void MainWindow::on_spinBoxNucDetThresholdingBaseline_valueChanged(const QString& arg)
{ if (arg.size() > 0) { on_spinBoxNucDetThresholdingBaseline_editingFinished(); } }

void MainWindow::on_radioButtonNucDetParamsAuto_toggled(bool checked)
{
  ui_->spinBoxNucDetThresholdingBlocksize->setEnabled(not checked);
  ui_->spinBoxNucDetThresholdingBaseline->setEnabled(not checked);

  ui_->groupBoxNucDetThresholdingBlocksizeAutoscanParams->setVisible(checked);
  ui_->groupBoxNucDetThresholdingBaselineAutoscanParams->setVisible(checked);
  ui_->groupBoxNucDetThresholdingBlocksizeAutoscanParams->setEnabled(checked);
  ui_->groupBoxNucDetThresholdingBaselineAutoscanParams->setEnabled(checked);

  ui_->labelNucDetThresholdingAutoscanObjective->setVisible(checked);
  ui_->comboBoxNucDetThresholdingAutoscanObjective->setVisible(checked);

  ui_->labelNucDetThresholdingThreadTimeout->setVisible(checked);
  ui_->dspinBoxNucDetThresholdingThreadTimeout->setVisible(checked);

  // without this check "over-threading" and endless loop
  if (checked) {
    maybe_NucDetDetect_autoclick();
    if (ui_->checkBoxNucDetDetectAutoup->isChecked()) {
      ui_->checkBoxNucDetDetectAutoup->setChecked(false);
  } }
#if 0
  struct Th : public QThread {
    Th(QFutureWatcher<img_preproc_scored_params_t>& w,QCheckBox* cb)
    : watcher_(w), checkbox_(cb) { }
    void run() { watcher_.waitForFinished(); checkbox_->setChecked(true); }
    QFutureWatcher<img_preproc_scored_params_t>& watcher_; QCheckBox* checkbox_;
  };
  auto* th = new Th(watcher_img_preproc_,ui_->checkBoxNucDetDetectAutoup);
  th->start();
#endif
}

void MainWindow::on_radioButtonNucDetParamsManual_toggled(bool checked)
{ on_radioButtonNucDetParamsAuto_toggled(not checked); }

void MainWindow::on_checkBoxNucDetEqualization_toggled(bool checked)
{
  ui_->spinBoxNucDetEqualizationThreshold->setEnabled(checked);
  maybe_NucDetDetect_autoclick();
}

void MainWindow::on_spinBoxNucDetEqualizationThreshold_valueChanged(QString const& a)
{ if (not a.isEmpty()){on_spinBoxNucDetEqualizationThreshold_editingFinished();} }

void MainWindow::on_spinBoxNucDetEqualizationThreshold_editingFinished()
{ maybe_NucDetDetect_autoclick(); }


// -- morphological opening --

void MainWindow::on_checkBoxNucDetOpening_toggled(bool /*checked*/)
{ maybe_NucDetDetect_autoclick(); }

void MainWindow::on_spinBoxNucDetOpeningTimes_editingFinished()
{ maybe_NucDetDetect_autoclick(); }

void MainWindow::on_spinBoxNucDetOpeningTimes_valueChanged(QString const& arg)
{ if (arg.size() > 0) {on_spinBoxNucDetOpeningTimes_editingFinished(); }  }


void MainWindow::on_spinBoxNucDetOpeningErosion_editingFinished()
{
  // // bind erosion and dilation kernel sizes:
  // ui_->spinBoxNucDetOpeningDilation
  //    ->setValue(ui_->spinBoxNucDetOpeningErosion->value());
  maybe_NucDetDetect_autoclick();
}

void MainWindow::on_spinBoxNucDetOpeningErosion_valueChanged(const QString& arg)
{ if (arg.size() > 0) { on_spinBoxNucDetOpeningErosion_editingFinished(); } }


void MainWindow::on_spinBoxNucDetOpeningDilation_editingFinished()
{
  // // bind erosion and dilation kernel sizes:
  // ui_->spinBoxNucDetOpeningErosion
  //    ->setValue(ui_->spinBoxNucDetOpeningDilation->value());
  maybe_NucDetDetect_autoclick();
}

void MainWindow::on_spinBoxNucDetOpeningDilation_valueChanged(const QString& arg)
{ if (arg.size() > 0) { on_spinBoxNucDetOpeningDilation_editingFinished(); } }


// -- morphological closing --

void MainWindow::on_checkBoxNucDetClosing_toggled(bool /*arg*/)
{ maybe_NucDetDetect_autoclick(); }


void MainWindow::on_spinBoxNucDetClosingDilation_editingFinished()
{
  ui_->spinBoxNucDetClosingErosion->setValue(
      ui_->spinBoxNucDetClosingDilation->value());
  maybe_NucDetDetect_autoclick();
}

void MainWindow::on_spinBoxNucDetClosingDilation_valueChanged(QString const& a)
{  if (a.size() > 0) { on_spinBoxNucDetClosingDilation_editingFinished(); }  }


void MainWindow::on_spinBoxNucDetClosingErosion_editingFinished()
{
  ui_->spinBoxNucDetClosingDilation->setValue(
      ui_->spinBoxNucDetClosingErosion->value());
  maybe_NucDetDetect_autoclick();
}

void MainWindow::on_spinBoxNucDetClosingErosion_valueChanged(QString const& a)
{ if (a.size() > 0) { on_spinBoxNucDetClosingErosion_editingFinished(); } }


void MainWindow::on_spinBoxNucDetClosingTimes_editingFinished()
{ maybe_NucDetDetect_autoclick();   }

void MainWindow::on_spinBoxNucDetClosingTimes_valueChanged(QString const &a)
{ if (a.size() > 0) { on_spinBoxNucDetClosingTimes_editingFinished(); } }


// == group box: edge detection ================================================

void MainWindow::on_spinBoxNucDetContourDetThresholdLow_editingFinished()
{ maybe_NucDetDetect_autoclick(); }

void MainWindow::on_spinBoxNucDetContourDetThresholdLow_valueChanged(QString const&a)
{ if (a.size()) { on_spinBoxNucDetContourDetThresholdLow_editingFinished(); } }


void MainWindow::on_dspinBoxNucDetContourDetThresholdsRatio_editingFinished()
{ maybe_NucDetDetect_autoclick(); }

void MainWindow::on_dspinBoxNucDetContourDetThresholdsRatio_valueChanged(QString const&a)
{ if (a.size()) on_dspinBoxNucDetContourDetThresholdsRatio_editingFinished(); }


void MainWindow::on_dspinBoxNucDetContourDetTolerance_editingFinished()
{ maybe_NucDetDetect_autoclick(); }

void MainWindow::on_dspinBoxNucDetContourDetTolerance_valueChanged(QString const&a)
{ if (a.size()) { on_dspinBoxNucDetContourDetTolerance_editingFinished(); } }

void MainWindow::on_checkBoxNucDetContourDetReconstruct_toggled(bool /*checked*/)
{ maybe_NucDetDetect_autoclick(); }


// == group box: nuclei assessment =============================================

// -- solidity --

void MainWindow::on_dspinBoxNucDetSolidity_editingFinished()
{ maybe_NucDetDetect_autoclick(); }

void MainWindow::on_dspinBoxNucDetSolidity_valueChanged(QString const& arg)
{ if (arg.size() > 0) { on_dspinBoxNucDetSolidity_editingFinished(); } }


// -- median size, min and max --

void MainWindow::on_dspinBoxNucDetMinAreaFrac_editingFinished()
{ maybe_NucDetDetect_autoclick(); }

void MainWindow::on_dspinBoxNucDetMinAreaFrac_valueChanged(QString const& arg)
{ if (arg.size() > 0) { on_dspinBoxNucDetMinAreaFrac_editingFinished(); } }

void MainWindow::on_dspinBoxNucDetMaxAreaFrac_editingFinished()
{ maybe_NucDetDetect_autoclick(); }

void MainWindow::on_dspinBoxNucDetMaxAreaFrac_valueChanged(QString const &arg)
{ if (arg.size() > 0) { on_dspinBoxNucDetMaxAreaFrac_editingFinished(); } }


// -- declumping adjacent contours --

void MainWindow::on_checkBoxNucDetSplitAdjacent_toggled(bool checked)
{
  ui_->labelNucDetSolidityMinPocketArea->setEnabled(checked);
  ui_->spinBoxNucDetMinPocketArea->setEnabled(checked);
  maybe_NucDetDetect_autoclick();
}

void MainWindow::on_spinBoxNucDetMinPocketArea_editingFinished()
{ maybe_NucDetDetect_autoclick(); }

void MainWindow::on_spinBoxNucDetMinPocketArea_valueChanged(QString const& arg)
{ if (arg.size() > 0) { on_spinBoxNucDetMinPocketArea_editingFinished(); } }


// == other ====================================================================

void MainWindow::on_checkBoxNucDetDetectAutoup_toggled(bool checked)
{
  ui_->pushButtonNucDetDetect->setText(
      QString("Detect nuclei") + (checked ? " (auto)" : ""));
  //ui_->pushButtonNucDetDetect->setDown(checked);
  //if (checked) { ui_->pushButtonNucDetDetect->click(); }
}
