﻿// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#include "MainWindow.hpp"
#include "ui_MainWindow.h"
#include "ZoomableGraphicsView.hpp"


QList<QAction*> MainWindow::channelPaneToggleActions_() const
{
  return {ui_->actionGreenChannelPanelToggle,
          ui_->actionRedChannelPanelToggle,
          ui_->actionBlueChannelPanelToggle,
          ui_->actionCyanChannelPanelToggle,
          ui_->actionMagentaChannelPanelToggle,
          ui_->actionYellowChannelPanelToggle,
          ui_->actionBrightfieldPanelToggle,
          ui_->actionOverlaysPanelToggle};
}


QList<QAction*> MainWindow::overlayChannelToggleActions_() const
{
  return {ui_->actionOverlaysGreenChannelInclusionToggle,
          ui_->actionOverlaysRedChannelInclusionToggle,
          ui_->actionOverlaysBlueChannelInclusionToggle,
          ui_->actionOverlaysCyanChannelInclusionToggle,
          ui_->actionOverlaysMagentaChannelInclusionToggle,
          ui_->actionOverlaysYellowChannelInclusionToggle,
          ui_->actionOverlaysBrightfieldInclusionToggle};
}


void MainWindow::maybeStoreViewSettings_()
{
  if (   views_.size() == 0
      or not ui_->actionPreferenceRetainViewForNextOpening->isChecked()) {
    return;
  }

  QList<QAction*> const pane_actions = channelPaneToggleActions_();
  for (QAction* a: pane_actions) {
    view_settings_.channel_pane_available.insert(a, a->isEnabled());
    view_settings_.channel_pane_visible  .insert(a, a->text() == "Hide");
  }

  QList<QAction*> const overlay_actions = overlayChannelToggleActions_();
  for (QAction* a: overlay_actions) {
    view_settings_.overlay_channel_available.insert(a, a->isEnabled());
    view_settings_.overlay_channel_included.insert(a, a->text().contains("Exclude"));
  }

  assert(views_.size() > 0);
  assert(qFuzzyCompare(views_[0]->transform().m11(),
                       views_[0]->transform().m22()));
  view_settings_.scale = views_[0]->transform().m11();
}


void MainWindow::maybeRestoreViewSettings_()
{
  if (   views_.size() == 0
      or not ui_->actionPreferenceRetainViewForNextOpening->isChecked()) {
    return;
  }

  bool panes_match = true;
  QList<QAction*> const pane_actions = channelPaneToggleActions_();
  for (QAction* a: pane_actions) {
    if (a->isEnabled() != view_settings_.channel_pane_available[a]) {
      panes_match = false;
    } else if (a->isEnabled() and view_settings_.channel_pane_available[a]) {
      if (   (     view_settings_.channel_pane_visible[a]  and (a->text() == "Show"))
          or ((not view_settings_.channel_pane_visible[a]) and (a->text() == "Hide"))) {
        a->trigger();
    } }
  } // for
  if (not panes_match) {
    message_('W', "Cannot retain exact visibility of channel panes "
                  "because of a mismatch of channels.");
  }

  bool overlay_match = true;
  QList<QAction*> const overlay_actions = overlayChannelToggleActions_();
  for (QAction* a: overlay_actions) {
    if (a->isEnabled() != view_settings_.overlay_channel_available[a]) {
      overlay_match = false;
    } else if (a->isEnabled() and view_settings_.overlay_channel_available[a]) {
      if (   (     view_settings_.overlay_channel_included[a]  and (a->text().contains("Include")))
          or ((not view_settings_.overlay_channel_included[a]) and (a->text().contains("Exclude")))) {
        a->trigger();
    } }
  } // for
  if (not overlay_match) {
    message_('W', "Cannot retain exact composition of channels in the overlay.");
  }

  for (auto& view: views_) {
    view->scale(view_settings_.scale,
                view_settings_.scale);
  }
}
