// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#ifndef SHUTTLETRACKER_GUI_WINDOWS_SCRIPTWINDOW_HPP
#define SHUTTLETRACKER_GUI_WINDOWS_SCRIPTWINDOW_HPP

#define SHUTTLETRACKER_GUI_EVAL_SCRIPT_IN_A_SEPARATE_THREAD 0

#include "GUI/GUICommon.hpp"
#include "GUI/Support/ScriptEditor.hpp"
#include <QWidget>
#include <QMainWindow>
#include <QJSEngine>
#include <QDir>

#if SHUTTLETRACKER_GUI_EVAL_SCRIPT_IN_A_SEPARATE_THREAD
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
#  include <QtConcurrent/QtConcurrent>
#  include <QtConcurrent/QtConcurrentRun>
#else
#  include <QtConcurrentRun>
#endif
#endif

namespace Ui { class ScriptWindow; }
class MainWindow;
class ScriptWindow : public QMainWindow
{
  Q_OBJECT

public:
  explicit ScriptWindow(MainWindow* main_window, QWidget* parent=nullptr);
  ~ScriptWindow() OVERRIDE;

  void setCompleterWordlist(QStringList const& words);

  QDir const& scriptsDir() const;
  QString const& scriptFileNameExtension() const;

  void seizeScriptFilesList()
  {  emit scriptFilesListModified(listAvailableScriptFiles_());  }

  int loadAndExecute(QString const& script_name);

  int execute(QString const& script);
  void abortExecution();

Q_SIGNALS:
  void scriptFilesListModified(QStringList);

protected:
  QStringList listAvailableScriptFiles_();
  void setupSnippets_();
  void preprocess_(QString&) const;

  void toggleMainWindowControl(bool);

private slots:
  void closeEvent(QCloseEvent* e) OVERRIDE {  hide();  e->ignore();  }

  void on_pushButtonScriptFileLoad_clicked();
  void on_pushButtonScriptFileSave_clicked();
  void on_pushButtonScriptFileNew_clicked();
  void on_pushButtonScriptFileDelete_clicked();

  void on_pushButtonScriptRun_clicked();
  void on_pushButtonSnippetsToggle_toggled(bool);

protected:
  QJSEngine interpreter_;
  MainWindow* main_window_;
  Ui::ScriptWindow* ui_;
  QString const main_window_str_ = "___MainWindow___";
  QString const script_file_ext_ = ".stscript";
  QDir scripts_dir_;
  bool last_evaluation_had_error_ = false;
#if SHUTTLETRACKER_GUI_EVAL_SCRIPT_IN_A_SEPARATE_THREAD
  QFuture<QScriptValue> script_future_;
#endif
}; // class ScriptWindow

#endif // !defined(SHUTTLETRACKER_GUI_WINDOWS_SCRIPTWINDOW_HPP)
