// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#ifndef SHUTTLETRACKER_IMAGE_ERRORHANDLING_HPP
#define SHUTTLETRACKER_IMAGE_ERRORHANDLING_HPP

#include "ImageCommon.hpp"
#include <stdexcept>

#ifdef USE_QT
//#  include <QObject>
//#  include <QDebug>
#  include <QtGlobal> // qt version macros
#  if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
//#  include <QtConcurrent/QtConcurrentRun>
//#  include <QtConcurrent/QtConcurrentMap>
#    include <QException>
#    define QT_EXCEPTION QException
#  else
#    include <QtConcurrentRun>
#    include <QtConcurrentMap>
#    define QT_EXCEPTION QtConcurrent::Exception
#  endif
#endif

namespace st {

#define THROW_ERROR(msg) \
  throw std::runtime_error(msg)

#ifdef USE_QT
struct QtInformativeException: public QT_EXCEPTION {
  QtInformativeException(std::exception&);
  void raise() const OVERRIDE;
  QT_EXCEPTION* clone() const OVERRIDE;
  const char* what() const NOEXCEPT OVERRIDE;
  std::exception error();
protected:
  std::exception err_;
  std::string what_;
};

#  define EXCEPTION_TRY \
  try

#  define EXCEPTION_RETHROW_SLAVE  \
  catch (std::exception&) { throw; }

#  define EXCEPTION_RETHROW_MASTER   \
  catch (std::exception& e) {        \
    progress()->yieldIdle();        \
    throw QtInformativeException(e); \
  }

#  define EXCEPTION_RETHROW          \
  catch (std::exception& e) {        \
    throw QtInformativeException(e); \
  }

#else
#  define EXCEPTION_TRY      /* */
#  define EXCEPTION_RETHROW_MASTER  /* */
#  define EXCEPTION_RETHROW_SLAVE   /* */
#endif // def USE_QT

} // namespace st

#endif // defined(SHUTTLETRACKER_IMAGE_ERRORHANDLING_HPP)
