// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#include "ImageChannel.hpp"
#include <algorithm>
#include <iterator>  // back_inserter
#include <cctype>
#include <string>

namespace st {

const std::string B_s = "blue";
const std::string G_s = "green";
const std::string R_s = "red";
const std::string C_s = "cyan";
const std::string M_s = "magenta";
const std::string Y_s = "yellow";
const std::string BF_s= "brightfield";
const std::string BF_s_LeicaLAS = "gray";
const std::string U_s = "undetermined";

// COMPILER note: initialization lists must be enveloped in oval brackets,
// otherwise static data remain uninitialized under VC++12 (MSVC2013).
const std::vector<ImageChannel> AllChannels
    ({ImageChannel::G,
      ImageChannel::R,
      ImageChannel::B,
      ImageChannel::C,
      ImageChannel::M,
      ImageChannel::Y,
      ImageChannel::BF});

std::map<ImageChannel, size_t> const TrichannelCvIndices
    ({{ImageChannel::B, 0},
      {ImageChannel::G, 1},
      {ImageChannel::R, 2}});


ImageChannel channelFrom(std::string const& name)
{
  std::string lowerized_name;
  std::transform(name.begin(), name.end(),
                 std::back_inserter(lowerized_name),
                 [](unsigned char c){ return std::tolower(c); });
  if      (lowerized_name == B_s) { return ImageChannel::B; }
  else if (lowerized_name == G_s) { return ImageChannel::G; }
  else if (lowerized_name == R_s) { return ImageChannel::R; }
  else if (lowerized_name == C_s) { return ImageChannel::C; }
  else if (lowerized_name == M_s) { return ImageChannel::M; }
  else if (lowerized_name == Y_s) { return ImageChannel::Y; }
  else if (lowerized_name == BF_s){ return ImageChannel::BF;}
  else if (lowerized_name == BF_s_LeicaLAS){ return ImageChannel::BF;}
  else { return ImageChannel::UNDETERMINED; }
}


std::string toString(ImageChannel channel)
{
  if      (channel == ImageChannel::B) { return B_s; }
  else if (channel == ImageChannel::G) { return G_s; }
  else if (channel == ImageChannel::R) { return R_s; }
  else if (channel == ImageChannel::C) { return C_s; }
  else if (channel == ImageChannel::M) { return M_s; }
  else if (channel == ImageChannel::Y) { return Y_s; }
  else if (channel == ImageChannel::BF){ return BF_s;}
  else { return U_s; }
}

} // namespace st
