// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#ifndef SHUTTLETRACKER_IMAGE_IMAGECHANNEL_HPP
#define SHUTTLETRACKER_IMAGE_IMAGECHANNEL_HPP

#include "ImageCommon.hpp"
#include <vector>
#include <map>
#include <string>

namespace st {

enum class ImageChannel { B,G,R, C,M,Y, BF, UNDETERMINED };

// all valid image channels
extern std::vector<ImageChannel> const AllChannels;

// OpenCV channel ordering
extern std::map<ImageChannel, size_t> const TrichannelCvIndices;

extern const std::string B_s;
extern const std::string G_s;
extern const std::string R_s;
extern const std::string C_s;
extern const std::string M_s;
extern const std::string Y_s;
extern const std::string BF_s;
extern const std::string BF_s_LeicaLAS;
extern const std::string U_s;

ImageChannel channelFrom(std::string const& text);
std::string toString(ImageChannel channel);

inline unsigned int toHash(ImageChannel channel)
{
  if      (channel == ImageChannel::B) { return 1<<0; }
  else if (channel == ImageChannel::G) { return 1<<1; }
  else if (channel == ImageChannel::R) { return 1<<2; }
  else if (channel == ImageChannel::C) { return 1<<3; }
  else if (channel == ImageChannel::M) { return 1<<4; }
  else if (channel == ImageChannel::Y) { return 1<<5; }
  else if (channel == ImageChannel::BF){ return 1<<6; }
  else { return 0; }
}

} // namespace st

#endif // defined(SHUTTLETRACKER_IMAGE_IMAGECHANNEL_HPP)
