// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#ifndef SHUTTLETRACKER_IMAGE_IMAGEOPERATIONS_HPP
#define SHUTTLETRACKER_IMAGE_IMAGEOPERATIONS_HPP

#include "ImageCommon.hpp"
#include "Types.hpp"
#include <opencv2/core.hpp>

namespace st {

void identity(cv::Mat&);

void normalize(cv::Mat& image);
void equalize(cv::Mat& image);
void adjustGamma(cv::Mat& image, double gamma);

void denoise(cv::Mat& image, double strength);
void denoiseBilateral(cv::Mat& image, double strength);
void smoothen(cv::Mat& img, int size_base);
void smoothenFixedSigma(cv::Mat& img, int size_base);

void adjustBrightnessContrast(cv::Mat& image, double alpha=1.0, int beta=0);
void brighten(cv::Mat& image);

void autolevel(cv::Mat& image, float low, float high);
void autolevelWeak(cv::Mat& image);
void autolevelMedium(cv::Mat& image);
void autolevelStrong(cv::Mat& image);

void threshold(cv::Mat& image, int block_size, int baseline = 0,
               cv::Mat const& xmask = cv::Mat{});

void morphologyOpen(cv::Mat& image, int ntimes, int erosion_size,
                                                int dilation_size);
void morphologyClose(cv::Mat& image, int ntimes, int dilation_size,
                                                 int erosion_size);

void dim(cv::Mat& image, cv::Mat const& xmask, double reduction, int blockSize,
         std::vector<int>const& sigmas = {5, 11, 21});

void blackout(cv::Mat& img, cv::Mat const& mask_img);

cv::Mat mergeWeighted(cv::Mat const& imageB, double weightB,
                      cv::Mat const& imageG = cv::Mat{}, double weightG = 0.,
                      cv::Mat const& imageR = cv::Mat{}, double weightR = 0.);

cv::Mat edgeImage(cv::Mat const& image,
                  bool preview = false,
                  float median_contour_length = 100.f,
                  int canny_threshold_low = 120,
                  float canny_thresholds_ratio = 2.f,
                  int canny_kernel_size = 5,
                  std::vector<std::pair<cv::Size, double>> const&
                      weighted_blur_kernels = {{{3, 3}, 0.55},
                                               {{5, 5}, 0.30},
                                               {{7, 7}, 0.15}},
                  int blur_kernel_range = 5,
                  double edges_weighting_by_gradient = 0.425,
                  double edges_prominence = 0.75);


namespace internal {

void adaptiveThresholdWithMask_(
    cv::Mat const& src, cv::Mat& dst, const cv::Mat& xmask, double max_value,
    int method, int type, int block_size, double delta);

} // namespace internal


namespace obsolete {

void desaturate3(cv::Mat& image);  // works on 3-channel images, poorly tested

} // namespace obsolete

} // namespace st

#endif // !defined(SHUTTLETRACKER_IMAGE_IMAGEOPERATIONS_HPP)
