// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#include "ImageSequenceFilenamePattern.hpp"
#include <string>
#include <boost/regex.hpp>

namespace st {

using ISFP = ImageSequenceFilenamePattern;

// export via Leica's LAS X
std::string const ISFP::re_leica_prefix_chunk_tc = "(.*(?=[TtZz][0-9]+))";
std::string const ISFP::re_leica_prefix_chunk_ct = "(.*(?=[Cc][Hh]?[0-9]+))";
std::string const ISFP::re_leica_timepoint_chunk = "_?[TtZz]([0-9]+)";
std::string const ISFP::re_leica_channel_chunk   = "_?[Cc][Hh]?([0-9]+)";

// export from Operetta's Harmony
std::string const ISFP::re_operetta_prefix_chunk    = "(r..c..f..p..-(?=ch[0-9]+sk[0-9]+fk[0-9]+fl[0-9]+))";
std::string const ISFP::re_operetta_channel_chunk   = "ch([0-9]+)";
std::string const ISFP::re_operetta_timepoint_chunk = "sk([0-9]+)";
std::string const ISFP::re_operetta_suffix_chunk    = "(fk[0-9]+fl[0-9]+)";

// common
std::string const ISFP::re_image_extension_chunk = "\\.(tiff?|TIFF?|png|PNG|jpe?g|JPE?G)";


boost::regex const ISFP::filename_pattern_leica_series_tc {
  std::string("^") + ISFP::re_leica_prefix_chunk_tc
                   + ISFP::re_leica_timepoint_chunk // |_ timepoint-then-channel
                   + ISFP::re_leica_channel_chunk   // |
                   + ISFP::re_image_extension_chunk + "$"
};
boost::regex const ISFP::filename_pattern_leica_series_ct {
  std::string("^") + ISFP::re_leica_prefix_chunk_ct
                   + ISFP::re_leica_channel_chunk   // |_ channel-then-timepoint
                   + ISFP::re_leica_timepoint_chunk // |
                   + ISFP::re_image_extension_chunk + "$"
};
boost::regex const ISFP::filename_pattern_leica_single{
  std::string("^") + ISFP::re_leica_prefix_chunk_ct
                   + ISFP::re_leica_channel_chunk
                   + ISFP::re_image_extension_chunk + "$"
};
boost::regex const ISFP::filename_pattern_operetta_series{
  std::string("^") + ISFP::re_operetta_prefix_chunk
                   + ISFP::re_operetta_channel_chunk
                   + ISFP::re_operetta_timepoint_chunk
                   + ISFP::re_operetta_suffix_chunk
                   + ISFP::re_image_extension_chunk + "$"
};

} // namespace st
