// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#ifndef SHUTTLETRACKER_IMAGE_NUCLEIDETECTIONSTAINED_HPP
#define SHUTTLETRACKER_IMAGE_NUCLEIDETECTIONSTAINED_HPP

#include "NucleiDetection.hpp"

namespace st {
namespace stained {

// In fact, to better reflect what could have happened to the image (depending
// in ImagePreprocessingParameters), every participle in the first group
// should contain 'MAYBE_'. The most truthfully/intuitively named enums seem
// to be those begining with 'BEFORE_'.
enum class ImagePreprocessingStage {
  NONE,
  //
  INTACT = NONE,
  //
  NORMALIZED,
  NORMALIZED_DENOISED,
  NORMALIZED_DENOISED_SMOOTHED,
  NORMALIZED_DENOISED_SMOOTHED_THRESHOLDED,
  NORMALIZED_DENOISED_SMOOTHED_THRESHOLDED_OPENED,
  //
  BEFORE_NORMALIZATION = INTACT,
  BEFORE_DENOISING     = NORMALIZED,
  BEFORE_SMOOTHING     = NORMALIZED_DENOISED,
  BEFORE_THRESHOLDING  = NORMALIZED_DENOISED_SMOOTHED,
  BEFORE_OPENING       = NORMALIZED_DENOISED_SMOOTHED_THRESHOLDED,
  //
  AFTER_NORMALIZATION = NORMALIZED,
  AFTER_DENOISING     = NORMALIZED_DENOISED,
  AFTER_SMOOTHING     = NORMALIZED_DENOISED_SMOOTHED,
  AFTER_THRESHOLDING  = NORMALIZED_DENOISED_SMOOTHED_THRESHOLDED,
  AFTER_OPENING       = NORMALIZED_DENOISED_SMOOTHED_THRESHOLDED_OPENED,
  //
  FINAL = NORMALIZED_DENOISED_SMOOTHED_THRESHOLDED_OPENED,
};

void preprocessImage(const cv::Mat& img_in,
                     ImagePreprocessingParameters const& params,
                     ImagePreprocessingStage stageA,  cv::Mat& img_outA,
                     ImagePreprocessingStage stageB,  cv::Mat& img_outB,
                     bool preview = false);

inline
void preprocessImage(const cv::Mat& img_in,
                     ImagePreprocessingParameters const& params,
                     ImagePreprocessingStage stage,  cv::Mat& img_out,
                     bool preview = false)
{
  cv::Mat dummy;
  preprocessImage(img_in, params, stage, img_out,
                  ImagePreprocessingStage::NONE, dummy, preview);
}

void findNuclearContours(cv::Mat const& img_preprocessed,
                        contours_vec_t& found,
                        bool preview = false);

void removeSpuriousNuclearContours(cv::Mat const& img_orig,
                                   st::contours_vec_t& contours,
                                   int peel_size = 3,      // PARAM
                                   double threshold = 0.); // PARAM
} // namespace stained_nuclei
} // namespace st

#endif // !defined(SHUTTLETRACKER_IMAGE_NUCLEIDETECTIONSTAINED_HPP)
