// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#ifndef SHUTTLETRACKER_IMAGE_PERINUCLEI_HPP
#define SHUTTLETRACKER_IMAGE_PERINUCLEI_HPP

#include "ImageCommon.hpp"
#include "Types.hpp"
#include "ImageChannel.hpp"
#include <opencv2/core/types.hpp>

namespace st {

struct Perinucleus {
  enum class Kind {kCompleteO, kCompleteC, kMultipartC, kEmpty};
  Kind kind;
  contours_vec_t contours;
};

struct Bicontour {
  contour_t inner;
  contour_t outer;
  cv::Point com;  // center of the mass of the inner contour
};

struct PerinucleiParameters {
  int dilation_in = 2;
  int dilation_out = 5;
  int neighbor_evasion = 2;
  int overlap_evasion = 3;
  int min_part_area = 30; // px

  bool  bg_evasion = false;
  ImageChannel bg_channel = ImageChannel::UNDETERMINED;
  float bg_expansion = 1.5f;
  int   bg_smoothing = 2;
};

void derivePerinuclei(std::vector<contour_t const*> const& nuclear_contours,
                      std::vector<Perinucleus>* perinuclei,
                      const cv::Mat& img_bg_mask,
                      cv::Size img_size,
                      PerinucleiParameters const& params);

void makeBackgroundMask(cv::Mat const& img_orig,
                        cv::Mat* dest_img_bg_mask,
                        PerinucleiParameters const& params,
                        std::pair<float, float> const&& bg_intens_mu_sigma);

} // namespace st

#endif // !defined(SHUTTLETRACKER_IMAGE_PERINUCLEI_HPP)
