// ShuttleTracker, a tracker of nucleocytoplasmic trafficking in cells
//
// Copyright (C)  2015-2023  Marek Kochanczyk <mkochan:at:ippt.pan.pl>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU GPL version 3. See LICENSE.md for details.

#ifndef SHUTTLETRACKER_IMAGE_SMALLUTILITIES_CPP
#define SHUTTLETRACKER_IMAGE_SMALLUTILITIES_CPP

#include "ImageCommon.hpp"
#include "Types.hpp"
#include <opencv2/core/mat.hpp>
#include <opencv2/imgproc.hpp>
#include <utility>  // std::pair

namespace st {

cv::Rect boundingBox(contour_t const& contour,
                     int add_margin = 0,
                     cv::Size image_size = cv::Size{0, 0});

cv::Point2f centerOfMass(contour_t const& contour);


inline void drawContour(cv::Mat& image,
                        contour_t const& contour,
                        int width = 1,
                        cv::Point shift = cv::Point{0, 0},
                        cv::Scalar brush = cv::Scalar{255})
{
  assert(width > 0);
  assert(not contour.empty());
  cv::drawContours(image, //contours_vec_t{contour},  // terse but copy can be
                   cv::Mat{1, static_cast<int>(contour.size()),  // avoided <--
                           CV_32SC2, const_cast<contour_t&>(contour).data()},
                   0, brush, width, cv::LINE_8,
                   cv::noArray(), std::numeric_limits<int>::max(), shift);
}


inline void drawMask(cv::Mat& image,
                     contour_t const& contour,
                     cv::Point shift = cv::Point{0, 0},
                     cv::Scalar brush = cv::Scalar{255})
{
  cv::drawContours(image, //contours_vec_t{contour},  // terse but copy can be
                   cv::Mat{1, static_cast<int>(contour.size()),  // avoided <--
                           CV_32SC2, const_cast<contour_t&>(contour).data()},
                   0, brush, cv::FILLED /*=>interior*/, cv::LINE_4,
                   cv::noArray(), std::numeric_limits<int>::max(), shift);
}


inline void drawMask(cv::Mat& image,
                     contours_vec_t const& contours, int contour_index,
                     cv::Point shift = cv::Point{0, 0},
                     cv::Scalar brush = cv::Scalar{255})
{
  cv::drawContours(image, contours, contour_index, brush,
                   cv::FILLED /*=>interior*/, cv::LINE_4,
                   cv::noArray(), std::numeric_limits<int>::max(), shift);
}


inline void drawMask(cv::Mat& image,
                     contours_vec_t const& contours,
                     cv::Point shift = cv::Point{0, 0},
                     cv::Scalar brush = cv::Scalar{255})
{
  cv::drawContours(image, contours, -1, brush,
                   cv::FILLED /*=>interior*/, cv::LINE_4,
                   cv::noArray(), std::numeric_limits<int>::max(), shift);
}


float edgeScore(cv::Mat const& image, contour_t const& contour);

} // namespace st

#endif // !defined(SHUTTLETRACKER_IMAGE_SMALLUTILITIES_CPP)
