#include "NucleiDetectionNonstained.hpp"
#include <opencv2/highgui.hpp>

using namespace std;
using namespace cv;
using namespace st;

Mat img_orig;
Mat img_pre_complete;
Mat img_pre_partial;
ImagePreprocessingParameters p0;
NucleiDetectionNonstainedParameters p1;

char const* window_name = "Non-stained nuclei contours detection";


void FindNuclei(int, void*)
{
  preprocessNonstainedNucleiImage(img_orig, img_pre_complete,
      img_pre_partial, p0, false);

  contours_vec_t cs;
  st::findNonstainedNucleiContours(img_orig, img_pre_complete,
      img_pre_partial, cs, p1, false);

  Mat img = img_orig.clone();
  cvtColor(img, img, COLOR_GRAY2BGR);
  drawContours(img, cs, -1, Scalar{32, 255, 255}, 1, LINE_AA);

  imshow(window_name, img);
  waitKey(0);
}


int main(int argc, char* argv[])
{
  img_orig = imread(argv[1]);
  if (!img_orig.data) { return -1; }
  cvtColor(img_orig, img_orig, COLOR_BGR2GRAY);

  p0.normalizing = true;
  p0.denoising = true;
  p0.denoising_strength = 4;
  p0.smoothing = true;
  p0.smoothing_size = 4;
  p0.thresholding = false;
//p0.thresholding_block_size;
//p0.thresholding_baseline;
  p0.opening = false;
//p0.opening_ntimes;
//p0.opening_erosion_size;
//p0.opening_dilation_size;
  p0.closing = true;
  p0.closing_ntimes = 1;
  p0.closing_dilation_size = 4;
  p0.closing_erosion_size  = 4;
 
  p1.threshold_low = 120;
  p1.thresholds_ratio = 4;
  p1.tolerance = 1.0;
  p1.reconstruct = true;

  namedWindow(window_name, WINDOW_AUTOSIZE);
  int smoothing_size_max = 12;
  createTrackbar("Smoothing:", window_name, &p0.smoothing_size,
      smoothing_size_max, FindNuclei);

  FindNuclei(0, 0);

  return 0;
}

